@@ -25,9 +25,9 @@ namespace acfg
 
 extern mstring cachedir, logdir, confdir, fifopath, user, group, pidfile, 
 reportpage, vfilepat, pfilepat, wfilepat, agentname, remoteport, adminauth, bindaddr, port, sUmask,
-dontcacherq, dontcachetgt, dontcache, mirrorsrcs, requestapx;
+tmpDontcacherq, tmpDontcachetgt, tmpDontcache, mirrorsrcs, requestapx;
 
-extern int debug, offlinemode, foreground, verbose, stupidfs, forcemanaged, 
+extern int debug, numcores, offlinemode, foreground, verbose, stupidfs, forcemanaged, keepnver,
 verboselog, extreshhold, exfailabort, tpstandbymax, tpthreadmax, dnscachetime, dlbufsize,
 exporigin, logxff, oldupdate, recompbz2, nettimeout, updinterval, forwardsoap, dirperms, fileperms;
 
@@ -35,6 +35,8 @@ exporigin, logxff, oldupdate, recompbz2, nettimeout, updinterval, forwardsoap, d
 extern tHttpUrl proxy_info;
 extern mstring agentheader;
 
+extern int conprotos[2];
+
 bool SetOption(const mstring &line, bool bQuiet=false);
 void ReadConfigDirectory(const char*);
 
@@ -47,21 +49,6 @@ void PostProcConfig();
 				
 const mstring * GetRepNameAndPathResidual(const tHttpUrl & in, mstring & sRetPathResidual);
 
-/*
-struct tBackendDesc : public tHttpUrl
-{
-	struct tHookHandler
-	{
-		virtual void PostponeDownCmd()=0;
-		virtual void UpCmd()=0;
-		virtual void PreUpCmd()=0;
-	};
-	tHookHandler *m_pHooks;
-	inline tBackendDesc(const tHttpUrl& src) : tHttpUrl(src), m_pHooks(NULL) {};
-	inline tBackendDesc() : m_pHooks(NULL) {};
-};
-*/
-
 typedef struct { 
 	const char *name; mstring *ptr;
 	const char *warn; }
@@ -84,11 +71,12 @@ struct tRepoData
 	};
 	IHookHandler *m_pHooks;
 	tStrVec m_keyfiles;
+	tHttpUrl m_deltasrc;
 	tRepoData() : m_pHooks(NULL) {};
 	virtual ~tRepoData();
 };
 
-const tRepoData * GetBackendVec(const mstring * vname);
+const tRepoData * GetBackendVec(cmstring &vname);
 
 time_t ExecutePostponed();
 
