@@ -20,7 +20,12 @@ struct foo;
 
 #define CACHE_BASE (acfg::cacheDirSlash)
 #define CACHE_BASE_LEN (CACHE_BASE.length()) // where the relative paths begin
+#define SZABSPATH(x) (CACHE_BASE+(x)).c_str()
+#define SABSPATH(x) (CACHE_BASE+(x))
 
+#define REDIRMAX 5
+
+#define FAKEDATEMARK "Sat, 26 Apr 1986 01:23:39 GMT+3"
 
 // caching all relevant file identity data and helper flags in such entries
 struct tDiskFileInfo
@@ -113,10 +118,10 @@ protected:
 		alreadyparsed:1, misc:1;
 		enumIndexType eIdxType:8;
 		const tStrDeq *bros;
-
-		tIfileAttribs() : vfile_ondisk(false), uptodate(false), parseignore(false),
+		off_t space;
+		inline tIfileAttribs() : vfile_ondisk(false), uptodate(false), parseignore(false),
 				hideDlErrors(false), forgiveDlErrors(false), alreadyparsed(false),
-				misc(false), eIdxType(EIDX_UNSUPPORTED), bros(NULL)
+				misc(false), eIdxType(EIDX_UNSUPPORTED), bros(NULL), space(0)
 		{};
 	};
 
@@ -150,18 +155,25 @@ protected:
 
 	void _ProcessSeenIndexFiles();
 
-	bool DownloadIdxFile(const mstring & sFilePathRel, mstring &sErrOut);
-	bool DownloadSimple(const mstring & sFilePathRel);
+
+	void StartDlder();
+
+	bool FindURL_InitDload(tFileItemPtr pFi, cmstring &cacheRefFileRel, mstring &sErr, tHttpUrl *pOrigURL);
+
+	class tGetItemDelegate;
+#define VERB_QUIET 0
+#define VERB_SHOW 1
+#define VERB_SHOW_NOERRORS 2
+	bool Download(const MYSTD::string & sFilePathRel, bool bIndexFile,
+			int nVerbosity=VERB_SHOW, tGetItemDelegate *p=NULL, const char *pForcedURL=NULL);
 
 	// internal helper variables
-	time_t m_nTimeNow;
-
 	bool m_bErrAbort, m_bVerbose, m_bForceDownload;
 	bool m_bScanInternals, m_bByPath, m_bByChecksum, m_bSkipHeaderChecks;
 	bool m_bNeedsStrictPathsHere, m_bTruncateDamaged;
 	int m_nErrorCount;
 
-	enumIndexType GuessIndexType(const mstring &sPath);
+	enumIndexType GuessIndexTypeFromURL(const mstring &sPath);
 
 	unsigned int m_nProgIdx, m_nProgTell;
 
@@ -172,6 +184,24 @@ protected:
 
 	MYSTD::map<mstring,bool> m_forcedTrashMap;
 
+	bool GetAndCheckHead(cmstring & sHeadfile, cmstring &sFilePathRel, off_t nWantedSize);
+	bool Inject(cmstring &fromRel, cmstring &toRel,
+			bool bSetIfileFlags=true, bool bUpdateRefdata=true,
+			const header *pForcedHeader=NULL, bool bTryLink=false);
+
+	void PrintStats(cmstring &title);
+	mstring m_processedIfile;
+
+	inline void ProgTell()
+	{
+		if (++m_nProgIdx == m_nProgTell)
+		{
+			SendFmt()<<"Scanning, found "<<m_nProgIdx<<" file"
+					<< (m_nProgIdx>1?"s":"") << "...<br />\n";
+			m_nProgTell*=2;
+		}
+	}
+
 private:
 	virtual void UpdateFingerprint(const mstring &sPathRel, off_t nOverrideSize,
 			uint8_t *pOverrideSha1, uint8_t *pOverrideMd5) =0;
@@ -184,16 +214,10 @@ private:
 			tPListConstIt pit, tPListConstIt itEnd,
 			const tFingerprint *verifData);
 	dlcon *m_pDlcon;
-	void StartDlder();
 	tContId2eqClass m_eqClasses;
 
-	bool GetAndCheckHead(cmstring & sHeadfile, cmstring &sFilePathRel, off_t nWantedSize);
-	bool Inject(cmstring &from, cmstring &to);
-
-	bool RefetchFile(tFileItemPtr pFi, cmstring &cacheRefFileRel, mstring &sErr);
-	bool RecDownload(tFileItemPtr pFi, const char *pURL, int nTTL=3);
-
 	bool IsDeprecatedArchFile(cmstring &sFilePathRel);
+
 };
 
 
@@ -202,4 +226,7 @@ static const string suxeWempty[] = { ".bz2", ".gz", ".lzma", ".xz", ""};
 static const string suxeByLhood[] = { "", ".bz2", ".gz", ".lzma", ".xz"};
 static const string suxeByCompSize[] = { ".xz", ".lzma", ".bz2", ".gz", ""};
 
+bool CompDebVerLessThan(cmstring &s1, cmstring s2);
+extern time_t m_gMaintTimeNow;
+
 #endif /*_CACHEMAN_H_*/
