@@ -60,23 +60,17 @@ class header {
       int LoadFromFile(const mstring & sPath);
       
       //! returns byte count or negative errno value
-      int StoreToFile(const mstring &sPath);
+      int StoreToFile(const mstring &sPath) const;
       
       void set(eHeadPos, const mstring &value);
       void set(eHeadPos, const char *val);
       void set(eHeadPos, const char *s, size_t len);
       void set(eHeadPos, off_t nValue);
       void del(eHeadPos);
-      //mstring getLike(const mstring &pattern);
-      //bool getUri(mstring & sRet);
       inline int getStatus() const { return frontLine.length()>9 ? atoi(frontLine.c_str()+8) : 500; };
-      //void SetStatusLine(mstring sLine);
-      //mstring const & get(const mstring &key) const;
-      //mstring as_string(bool bTerminateIt) const;
-      //void strip();
       void clear();
       
-      mstring ToString();
+      mstring ToString() const;
 
    private:
 	   int Load(const char *src, UINT length);
