@@ -238,6 +238,11 @@ inline off_t atoofft(const char *p)
 	return atol(p);
 }
 
+inline off_t atoofft(const char *p, off_t nDefVal)
+{
+	return p ? atoofft(p) : nDefVal;
+}
+
 mstring offttosH(off_t n);
 
 tStrDeq ExpandFilePattern(const mstring &pattern, bool bSorted=false);
@@ -290,10 +295,20 @@ inline mstring offttosH(off_t n)
 	return "INF";
 }
 
+inline void replaceChars(mstring &s, const char *szBadChars, char goodChar)
+{
+	for(mstring::iterator p=s.begin();p!=s.end();p++)
+		for(const char *b=szBadChars;*b;b++)
+			if(*b==*p)
+			{
+				*p=goodChar;
+				break;
+			}
+}
+
 extern cmstring sEmptyString;
 
 //! split-and-extract helper for strings, for convenient use with for-loops
-// Only for local application, uses references!
 class tSplitWalk
 {
 	cmstring &s;
@@ -301,29 +316,41 @@ class tSplitWalk
 	const char *m_seps;
 
 public:
-	inline tSplitWalk(cmstring &line, const char *separators=SPACECHARS)
-	: s(line), start(0), len(stmiss), oob(line.size()), m_seps(separators) {}
+	inline tSplitWalk(cmstring *line, const char *separators=SPACECHARS)
+	: s(*line), start(0), len(stmiss), oob(line->size()), m_seps(separators) {}
 	inline bool Next()
 	{
 		if(len != stmiss) // initial state
 			start = start + len + 1;
 		if(start>=oob)
 			return false;
-		start = s.find_first_not_of(m_seps, start);
-		if(start>=oob)
-			return false;
 
-		len = s.find_first_of(m_seps, start);
-		if(len>=oob)
-			len=oob-start;
-		else
-			len-=start;
+		start = s.find_first_not_of(m_seps, start);
+
+		if(start<oob)
+		{
+			len = s.find_first_of(m_seps, start);
+			len = (len == stmiss) ? oob-start : len-start;
+		}
+		else if (len != stmiss) // not initial state, end reached
+			return false;
+		else if(s.empty()) // initial state, no parts
+			return false;
+		else // initial state, use the whole string
+		{
+			start = 0;
+			len = oob;
+		}
 
 		return true;
 	}
-	inline mstring GetPart(){ return s.substr(start, len); }
-	inline operator mstring() { return GetPart(); }
+	inline mstring str(){ return s.substr(start, len); }
+	inline operator mstring() { return str(); }
 };
 
+//bool CreateDetachedThread(void *(*threadfunc)(void *));
+
+void DelTree(cmstring &what);
+
 #endif // _META_H
 
