@@ -38,10 +38,9 @@ namespace acfg {
 
 
 // internal stuff:
-char alphabet[65] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 string sPopularPath("/debian/");
 
-string dontcache, dontcacherq, dontcachetgt;
+string tmpDontcache, tmpDontcacherq, tmpDontcachetgt, tmpConprotoparm;
 
 tStrMap localdirs;
 NoCaseStringMap mimemap;
@@ -62,11 +61,12 @@ MapNameToString n2sTbl[] = {
 		,{ "AdminAuth",  &adminauth, 0}
 		,{ "BindAddress", &bindaddr, 0}
 		,{ "UserAgent", &agentname, 0}
-		,{ "DontCache",	&dontcache, 0}
-		,{ "DontCacheRequested",	&dontcachetgt, 0}
-		,{ "DontCacheResolved",	&dontcacherq, 0}
+		,{ "DontCache",	&tmpDontcache, 0}
+		,{ "DontCacheRequested",	&tmpDontcachetgt, 0}
+		,{ "DontCacheResolved",	&tmpDontcacherq, 0}
 		,{ "PrecacheFor", &mirrorsrcs, 0}
 		,{ "RequestAppendix", &requestapx, 0}
+		,{ "ConnectProto", &tmpConprotoparm, 0}
 };
 
 MapNameToInt n2iTbl[] = {
@@ -91,6 +91,7 @@ MapNameToInt n2iTbl[] = {
 		,{ "NetworkTimeout", &nettimeout, 0, 10}
 		,{ "MinUpdateInterval", &updinterval, 0, 10}
 		,{ "ForwardBtsSoap", &forwardsoap, 0, 10}
+		,{ "KeepExtraVersions", &keepnver, 0, 10}
 
 		,{ "DirPerms", &dirperms, 0, 8}
 		,{ "FilePerms", &fileperms, 0, 8}
@@ -251,16 +252,28 @@ inline void AddRemapFlag(const string & token, const string &repname)
 	if(!ParseOptionLine(token, key, value, true))
 		return;
 
-	if(key=="keyfile" && !value.empty())
+	if(value.empty())
+		return;
+
+	tRepoData &where = repoparms[repname];
+
+	if(key=="keyfile")
 	{
 		//if(!startsWithSz(value, "/"))
 		//	value.insert(0, "/");
 		if (acfg::debug>1)
 			cerr << "Fatal keyfile for " <<repname<<": "<<value <<endl;
 
-		tRepoData &where = repoparms[repname];
 		where.m_keyfiles.push_back(value);
 	}
+	else if(key=="deltasrc")
+	{
+		if(!endsWithSzAr(value, "/"))
+			value+="/";
+
+		if(!where.m_deltasrc.SetHttpUrl(value))
+			cerr << "Couldn't parse Debdelta source URL, ignored " <<value <<endl;
+	}
 }
 
 inline void AddRemapInfo(bool bAsBackend, const string & token,
@@ -367,6 +380,7 @@ void _AddHooksFile(cmstring& vname)
 string EncodeBase64Auth(const string & s)
 {
 	int cols=0, bits=0, c=0, char_count=0;
+	char alphabet[65] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 	tStrPos pos=0;
 	string out;
 	while ( pos<s.size() )
@@ -417,9 +431,9 @@ string EncodeBase64Auth(const string & s)
 
 inline void _ParseLocalDirs(cmstring &value)
 {
-	for(tSplitWalk splitter(value, ";"); splitter.Next(); )
+	for(tSplitWalk splitter(&value, ";"); splitter.Next(); )
 	{
-		mstring token=splitter.GetPart();
+		mstring token=splitter.str();
 		trimString(token);
 		tStrPos pos = token.find_first_of(SPACECHARS);
 		if(stmiss == pos)
@@ -450,7 +464,7 @@ inline void _FetchMimeTypes()
 
 	while(mreader.GetOneLine(line))
 	{
-		tSplitWalk split(line);
+		tSplitWalk split(&line);
 		if(!split.Next())
 			continue;
 
@@ -634,11 +648,9 @@ const string * GetRepNameAndPathResidual(const tHttpUrl & in, string & sRetPathR
 	
 }
 
-const tRepoData * GetBackendVec(const string * vname)
+const tRepoData * GetBackendVec(cmstring &vname)
 {
-	if(!vname)
-		return NULL;
-	tRepMap::iterator it=repoparms.find(*vname);
+	tRepMap::iterator it=repoparms.find(vname);
 	if(it==repoparms.end() || it->second.m_backends.empty())
 		return NULL;
 	return & it->second;
@@ -958,14 +970,43 @@ void PostProcConfig()
    if(RESERVED_DEFVAL == forwardsoap)
 	   forwardsoap = !forcemanaged;
 
+#ifdef _SC_NPROCESSORS_ONLN
+	numcores = (int) sysconf(_SC_NPROCESSORS_ONLN);
+#elif defined(_SC_NPROC_ONLN)
+	numcores = (int) sysconf(_SC_NPROC_ONLN);
+#endif
 
-   if(!rechecks::CompileUncExpressions(dontcachetgt.empty() ? dontcache : dontcachetgt,
-		   dontcacherq.empty() ? dontcache : dontcacherq))
+   if(!rechecks::CompileUncExpressions(tmpDontcachetgt.empty() ? tmpDontcache : tmpDontcachetgt,
+		   tmpDontcacherq.empty() ? tmpDontcache : tmpDontcacherq))
    {
-	   cerr << "An error occured while compiling regular expression for non-cached paths!" <<endl;
+	   cerr << "An error occurred while compiling regular expression for non-cached paths!" <<endl;
 	   exit(EXIT_FAILURE);
    }
 
+   int *p=conprotos;
+   for(tSplitWalk split(&tmpConprotoparm); split.Next(); ++p)
+   {
+	   cmstring val(split);
+	   if(val.empty())
+		   break;
+
+	   if(p>=conprotos+_countof(conprotos))
+	   {
+		   cerr << "Too many protocols specified: " << val <<endl;
+		   exit(EXIT_FAILURE);
+	   }
+
+	   if(val == "v6")
+		   *p=PF_INET6;
+	   else if(val == "v4")
+		   *p=PF_INET;
+	   else
+	   {
+		   cerr << "IP protocol not supported: " << val <<endl;
+		   exit(EXIT_FAILURE);
+	   }
+   }
+
    if (acfg::debug>=LOG_MORE)
 	{
 		for (UINT i=0; i<_countof(n2sTbl); i++)
@@ -1169,7 +1210,7 @@ inline bool CompileUncachedRex(const string & token, bool bIsTgtPattern, bool bR
 bool CompileUncExpressions(const string & req, const string & tgt)
 {
 	for(int i=0; i<2; ++i) // req, tgt, stop
-		for(tSplitWalk split(i? req : tgt); split.Next(); )
+		for(tSplitWalk split(i? &req : &tgt); split.Next(); )
 			if (!CompileUncachedRex(split, i, false))
 				return false;
 	return true;
