@@ -24,7 +24,32 @@ void term_handler(int signum);
 void log_handler(int signum);
 void check_algos();
 
-int main(int argc, char **argv) {
+#ifndef HAVE_DAEMON // provide a poor but sufficient replacement
+int daemon(int, int)
+{
+   chdir("/");
+   int dummy=open("/dev/null", O_RDWR);
+   if(0<=dup2(dummy, fileno(stdin))
+		   && 0<=dup2(dummy, fileno(stdout))
+		   && 0<=dup2(dummy, fileno(stderr)))
+   {
+	   switch(fork())
+	   {
+	   case 0: // this is child, good
+		   return 0;
+	   case -1: // bad...
+		   return -1;
+	   default: // in parent -> cleanup
+		   setsid();
+		   _exit(0);
+	   }
+   }
+   return -1;
+}
+#endif
+
+int main(int argc, char **argv)
+{
 
 
 // PLAYGROUND
@@ -141,10 +166,11 @@ int main(int argc, char **argv) {
         cerr << "Problem creating log files. Check permissions of the log directory, " << acfg::logdir<<endl;
         exit(1);
     }
+
+	acfg::PostProcConfig();
     
     _TestCacheDir();
 
-	acfg::PostProcConfig();
 	conserver::Setup();
 	
 	if (bForceCleanup) {
@@ -194,24 +220,23 @@ static void usage() {
 }
 
 
-static void _TestCacheDir() {       
-	
+static void _TestCacheDir()
+{
 	struct timeval tv;
 	gettimeofday(&tv, NULL);
-	
-	vector<char> buf(acfg::cachedir.size()+30);
-	snprintf(&buf[0], buf.size(), "%s"SZPATHSEP"testfile.%0lx",
-			acfg::cachedir.c_str(), 42*tv.tv_usec*tv.tv_sec);
-	
-	mkbasedir(&buf[0]); // try or force its directory creation
-	int t=open(&buf[0], O_WRONLY | O_CREAT | O_TRUNC, 00644);
-	if (t<0) {
-		cerr << "Cache directory not writable. Check the permissions of "
-				<< acfg::cachedir << "!" <<endl;
-		exit(1);
+	tSS buf;
+	buf << acfg::cacheDirSlash << "testfile." <<42*tv.tv_usec*tv.tv_sec;
+	mkbasedir(buf.c_str()); // try or force its directory creation
+	int t=open( buf.c_str(), O_WRONLY | O_CREAT | O_TRUNC, 00644);
+	if (t>=0)
+	{
+		forceclose(t);
+		if(0==unlink(buf.c_str()))
+			return;
 	}
-	close(t);
-	unlink(&buf[0]);
+	cerr << "Cache directory not writable. Check the permissions of "
+		<< acfg::cachedir << "!" <<endl;
+	exit(1);
 }
 
 void log_handler(int)
@@ -219,8 +244,6 @@ void log_handler(int)
 	aclog::close(true);
 }
 
-char *crashMe(NULL);
-
 void term_handler(int signum)
 {
 	switch (signum)
@@ -229,7 +252,10 @@ void term_handler(int signum)
 	case (SIGINT):
 	case (SIGQUIT):
 		aclog::close(false);
+	// and then terminate...
 #if 0
+		{
+		char *crashMe(NULL);
 		// work is done, commit suicide
 		struct sigaction act;
 		memset(&act, 0, sizeof(act));
@@ -237,8 +263,11 @@ void term_handler(int signum)
 		if (sigaction(SIGINT, &act, NULL))
 			*crashMe = signum; // be sure not to end up in an infinite loop
 		raise(SIGINT);
+		}
 #endif
+#ifdef DEBUG
 	case SIGUSR2:
+#endif
 		exit(1);
 	default:
 		return;
