@@ -27,14 +27,14 @@
 
 using namespace MYSTD;
 
-#define SZABSPATH(x) (CACHE_BASE+(x)).c_str()
-#define SABSPATH(x) (CACHE_BASE+(x))
 static cmstring diffIdxSfx(".diff/Index");
 static cmstring sPatchBaseRel("_actmp/patch.base");
 static cmstring sPatchResRel("_actmp/patch.result");
+static unsigned int nKillLfd=1;
+time_t m_gMaintTimeNow=0;
 
 tCacheMan::tCacheMan(int fd) :
-	tWuiBgTask(fd), m_nTimeNow(0),
+	tWuiBgTask(fd),
 	m_bErrAbort(false), m_bVerbose(false), m_bForceDownload(false),
 	m_bScanInternals(false), m_bByPath(false), m_bByChecksum(false), m_bSkipHeaderChecks(false),
 	m_bNeedsStrictPathsHere(false), m_bTruncateDamaged(false),
@@ -42,7 +42,7 @@ tCacheMan::tCacheMan(int fd) :
 	m_nProgIdx(0), m_nProgTell(1), m_pDlcon(NULL)
 {
 	m_szDecoFile="maint.html";
-	m_nTimeNow=time(NULL);
+	m_gMaintTimeNow=time(NULL);
 
 }
 
@@ -52,6 +52,12 @@ tCacheMan::~tCacheMan()
 	m_pDlcon=NULL;
 }
 
+class tCacheMan::tGetItemDelegate
+{
+public:
+	virtual operator tFileItemPtr()=0;
+};
+
 bool tCacheMan::ProcessOthers(const string & sPath, const struct stat &)
 {
 	// NOOP
@@ -71,7 +77,7 @@ void tCacheMan::AddIFileCandidate(const string &sPathRel)
 	if ( (FILE_INDEX == rechecks::GetFiletype(sPathRel)
 	// SUSE stuff, not volatile but also contains file index data
 	|| endsWithSzAr(sPathRel, ".xml.gz") )
-	&& (t=GuessIndexType(sPathRel)) != EIDX_UNSUPPORTED)
+	&& (t=GuessIndexTypeFromURL(sPathRel)) != EIDX_UNSUPPORTED)
 	{
 		tIfileAttribs & atts=m_indexFilesRel[sPathRel];
 		atts.vfile_ondisk=true;
@@ -93,8 +99,8 @@ tCacheMan::tIfileAttribs &tCacheMan::SetFlags(cmstring &sPathRel)
 	return m_indexFilesRel[sPathRel];
 }
 
-
-bool tCacheMan::RecDownload(tFileItemPtr pFi, const char *pURL, int nTTL)
+/*
+bool tCacheMan::DownloadRec(tFileItemPtr pFi, const char *pURL, int nTTL)
 {
 	if(--nTTL<0)
 	{
@@ -111,19 +117,26 @@ bool tCacheMan::RecDownload(tFileItemPtr pFi, const char *pURL, int nTTL)
 		return false;
 	m_pDlcon->AddJob(pFi, url);
 	m_pDlcon->WorkLoop();
-	switch (pFi->GetHeader().getStatus())
+
+	int httpCode(0);
+	if(FIST_COMPLETE != pFi->WaitForFinish(&httpCode))
+		return false;
+
+	switch (httpCode)
 	{
 	case 200:
 		return true;
 	case 301:
 	case 302:
-		return RecDownload(pFi, pFi->GetHeader().h[header::LOCATION]);
+		return DownloadRec(pFi, pFi->GetHeader().h[header::LOCATION]);
 	default:
 		return false;
 	}
 }
+*/
 
-bool tCacheMan::RefetchFile(tFileItemPtr fi, cmstring &sFilePathRel, mstring &sErr)
+bool tCacheMan::FindURL_InitDload(tFileItemPtr fi, cmstring &sFilePathRel,
+		mstring &sErr, tHttpUrl *retURL)
 {
 	StartDlder();
 	if(!m_pDlcon)
@@ -151,21 +164,23 @@ bool tCacheMan::RefetchFile(tFileItemPtr fi, cmstring &sFilePathRel, mstring &sE
 		return false;
 	}
 
-	pBackends = acfg::GetBackendVec(&url.sHost);
+	string sOrig;
+	{
+		lockguard g(fi.get());
+		const header & pHead=fi->GetHeaderUnlocked();
+		if (pHead.h[header::XORIG])
+			sOrig=pHead.h[header::XORIG];
+	}
+
+	pBackends = acfg::GetBackendVec(url.sHost);
 
 	if (pBackends)
 		// HIT, control by backend scheme, strip leading slash from path
 		 m_pDlcon->AddJob(fi, pBackends, url.sPath.substr(1));
-	else
+
+	if(!pBackends || retURL)
 	{
 		tHttpUrl urlOrig;
-		string sOrig;
-		{
-			lockguard g(fi.get());
-			const header & pHead=fi->GetHeaderUnlocked();
-			if (pHead.h[header::XORIG])
-				sOrig=pHead.h[header::XORIG];
-		}
 		if(startsWithSz(sOrig, "http://") && urlOrig.SetHttpUrl(sOrig))
 		{
 			// ok, looks valid, is it better than the one from the path?
@@ -173,16 +188,10 @@ bool tCacheMan::RefetchFile(tFileItemPtr fi, cmstring &sFilePathRel, mstring &sE
 			{
 				if(urlOrig.sHost.find(".") != stmiss)
 				{
-					if(url.sHost.find(".") != stmiss)
+					if(url.sHost.find(".") == stmiss)
 					{
-						// Both have dots, prefer directory as host
-						//goto dl_from_url;
-					}
-					else
-					{
-						// dir has no dots, orig-url host has -> use orig url
+						// dir has no dots, x-orig-url host has -> prefer x-orig url
 						url=urlOrig;
-						//goto dl_from_url;
 					}
 				}
 				else // no dots in urlOrig host, most likely broken
@@ -200,12 +209,15 @@ bool tCacheMan::RefetchFile(tFileItemPtr fi, cmstring &sFilePathRel, mstring &sE
 				}
 			}
 		}
-		m_pDlcon->AddJob(fi, url);
 	}
 
-	m_pDlcon->WorkLoop();
+	if(!pBackends)
+		m_pDlcon->AddJob(fi, url);
 
-	return FIST_ERROR != fi->GetStatus(); // this is precise enough for this multi-purpose function, caller might need to recheck it against the own expectations
+	if(retURL)
+		*retURL = url;
+
+	return true;
 }
 
 bool tCacheMan::IsDeprecatedArchFile(cmstring &sFilePathRel)
@@ -250,15 +262,14 @@ bool tCacheMan::IsDeprecatedArchFile(cmstring &sFilePathRel)
 
 		while(reader.GetOneLine(sLine))
 		{
-			tSplitWalk w(sLine, SPACECHARS);
-			if(!w.Next() || w.GetPart() != "Architectures:")
+			tSplitWalk w(&sLine, SPACECHARS);
+			if(!w.Next() || w.str() != "Architectures:")
 				continue;
 			while(w.Next())
 			{
-				if(sFilePathRel.compare(pos, posend-pos, w.GetPart()) == 0)
+				if(sFilePathRel.compare(pos, posend-pos, w.str()) == 0)
 					return false; // architecture is there, not deprecated
 			}
-#warning fixme, und nu, wie kommt das in die loeschliste?
 			return true; // okay, now that arch should have been there :-(
 		}
 	}
@@ -267,155 +278,160 @@ bool tCacheMan::IsDeprecatedArchFile(cmstring &sFilePathRel)
 }
 
 
-static unsigned int nKillLfd=1;
-
-bool tCacheMan::DownloadIdxFile(const MYSTD::string & sFilePathRel, string &sErr)
+bool tCacheMan::Download(const MYSTD::string & sFilePathRel, bool bIndexFile,
+		int nVerbosity, tGetItemDelegate *pItemDelg, const char *pForcedURL)
 {
-	sErr.clear();
+	mstring sErr;
 	bool bSuccess=false;
-	tFileItemPtr fi;
 
 #ifdef DEBUG
-	bool nix=StrHas(sFilePathRel, "etch-unikl");
+	bool nix=StrHas(sFilePathRel, "debrep/dists/experimental/non-free/debian-installer/binary-mips/Packages.gz");
 #endif
 
 	const tIfileAttribs &flags=GetFlags(sFilePathRel);
 	if(flags.uptodate)
 	{
-		SendFmt()<<"Checking "<<sFilePathRel<<"... (fresh)<br>\n";
+		if(nVerbosity)
+			SendFmt()<<"Checking "<<sFilePathRel<< (bIndexFile
+					? "... (fresh)<br>\n" : "... (complete)<br>\n");
 		return true;
 	}
 
-	fi=fileitem::GetFileItem(sFilePathRel);
+#define GOTOREPMSG(x) {sErr = x; bSuccess=false; if(fi) {fi->Unreg(); fi.reset();} goto rep_dlresult; }
 
-	if (fi)
+	tHttpUrl origURL;
+	for(int i=0; i<=REDIRMAX && !bSuccess; ++i)
 	{
-		fi->Setup(true);
-		if(m_bForceDownload) // setup first and drop the contents then
+		tFileItemPtr fi;
+		// handle alternative behavior in the first or last cycles
+		if(0==i && pForcedURL)
 		{
-			fi->ResetCacheState();
-			SendFmt()<<"Downloading "<<sFilePathRel<<"...\n";
+			if(!origURL.SetHttpUrl(pForcedURL))
+				return false;
+			continue;
+		}
+		if(i==REDIRMAX)
+			GOTOREPMSG("too many redirections.");
+
+		fi = pItemDelg ? *pItemDelg : fileitem::GetRegisteredFileItem(sFilePathRel);
+
+		if(!fi)
+		{
+			if(nVerbosity)
+				SendFmt()<<"Checking "<<sFilePathRel<<"...\n"; // just display the name ASAP
+			GOTOREPMSG(" could not create file item handler.");
+		}
+		if(bIndexFile && m_bForceDownload)
+		{
+			if(!fi->SetupClean())
+				GOTOREPMSG("Item busy, cannot reload");
+			if(nVerbosity)
+				SendFmt()<<"Downloading "<<sFilePathRel<<"...\n";
 		}
 		else
 		{
-			SendFmt()<<"Checking/Updating "<<sFilePathRel<<"...\n";
+			FiStatus initState = fi->Setup(bIndexFile);
+			if(FIST_COMPLETE == initState && !bIndexFile)
+			{
+				SendFmt()<<"Checking "<<sFilePathRel<<"... (complete)<br>\n";
+				return true;
+			}
+			if(nVerbosity)
+				SendFmt()<< (bIndexFile?"Checking/Updating " : "Downloading ")
+				<<sFilePathRel<<"...\n";
 		}
-	}
-	else
-	{
-		SendFmt()<<"Checking "<<sFilePathRel<<"...\n"; // just display the name
 
-		sErr=" could not create file item handler.";
-		goto rep_dlresult;
-	}
-
-
-#ifdef WIN32
-#error rewrite for the backslashes or change reliably in the walker to slashes
-#endif
-
-	if (RefetchFile(fi, sFilePathRel, sErr))
-	{
-		int code=fi->GetHeader().getStatus();
-		if(301==code || 302==code)
+		if(i == 0)
 		{
-			RecDownload(fi, fi->GetHeader().h[header::LOCATION]);
-			code=fi->GetHeader().getStatus();
-		}
-		if(200==code)
-		{
-			bSuccess=true;
-			SendFmt() << "<i>(" << fi->GetTransferCount() / 1024 << "KiB)</i>\n";
-		}
-		else if(IsDeprecatedArchFile(sFilePathRel))
-		{
-			SendFmt() << "<i>(no longer available)</i>\n";
-			m_forcedTrashMap[sFilePathRel] = true;
-			bSuccess=true;
+			if( ! FindURL_InitDload(fi, sFilePathRel, sErr, &origURL))
+				GOTOREPMSG("Failed to resolve source URL");
 		}
+		else if(origURL.sHost.empty())
+			GOTOREPMSG("Failed to resolve original URL")
 		else
+			m_pDlcon->AddJob(fi, origURL);
+
+		m_pDlcon->WorkLoop();
+		int httpCode;
+		if(fi->WaitForFinish(&httpCode) != FIST_COMPLETE)
+			httpCode=555;
+
+		switch(httpCode)
 		{
-			bSuccess = flags.forgiveDlErrors;
-			if(bSuccess)
-				SendFmt() << "<i>(ignored)</i>\n";
+		case 200:
+			bSuccess=true;
+			if(nVerbosity)
+				SendFmt() << "<i>(" << fi->GetTransferCount() / 1024 << "KiB)</i>\n";
+			fi->Unreg(); // anyhow, release our pointer
+			fi.reset();
+			break;
+		case 301:
+		case 302:
+		{
+			header h=fi->GetHeader();
+			fi->Unreg(); // anyhow, release our pointer
+			fi.reset();
+			if(h.h[header::LOCATION] && origURL.SetHttpUrl(h.h[header::LOCATION]))
+				continue;
+		}
+		default:
+		{
+			if (IsDeprecatedArchFile(sFilePathRel))
+			{
+				if(nVerbosity == VERB_SHOW)
+					SendFmt() << "<i>(no longer available)</i>\n";
+				m_forcedTrashMap[sFilePathRel] = true;
+				bSuccess = true;
+			}
+			else if (flags.forgiveDlErrors)
+			{
+				bSuccess = true;
+				if(nVerbosity == VERB_SHOW)
+					SendFmt() << "<i>(ignored)</i>\n";
+			}
+			else
+				GOTOREPMSG(fi->GetHttpMsg());
+
+			fi->Unreg(); // anyhow, release our pointer
+			fi.reset();
+			break;
+		}
 		}
 	}
-#ifdef DEBUG
-	else
-	{
-		bSuccess=false;
-	}
-#endif
 
 	rep_dlresult:
 
-	if (bSuccess)
+	if (bSuccess && bIndexFile)
 		SetFlags(sFilePathRel).uptodate=true;
-	else if (fi)
-		sErr += fi->GetHttpMsg();
-	else if(sErr.empty())
-		sErr += "Download error";
 
 	UpdateFingerprint(sFilePathRel, -1, NULL, NULL);
 
-	if (fi)
-		fi->Unreg();
-
 	if(m_bVerbose || !bSuccess)
 	{
 		m_bShowControls=true;
-		SendFmt() << "<label><input type=\"checkbox\" name=\"kf"
-		<< (nKillLfd++) << "\" value=\"" << sFilePathRel
-		<< "\">Tag</label>";
+
+		if(!bSuccess)
+		{
+			if(nVerbosity == VERB_SHOW)
+			{
+				if(sErr.empty())
+					sErr = "Download error";
+				SendFmt() << "<font color=\"red\"><b>" << sErr << "</b></font>\n";
+			}
+		}
+
+		if(nVerbosity == VERB_SHOW)
+			SendFmt() << "<label><input type=\"checkbox\" name=\"kf"
+			<< (nKillLfd++) << "\" value=\"" << sFilePathRel
+			<< "\">Tag</label>";
 	}
 
-	SendChunk("<br>\n");
+	if(nVerbosity!=VERB_QUIET)
+		SendChunk("<br>\n");
+
 	return bSuccess;
 }
 
-
-bool tCacheMan::DownloadSimple(const MYSTD::string & sFilePathRel)
-{
-	bool bSuccess=false;
-	FiStatus fistate=FIST_FRESH;
-	tFileItemPtr fi=fileitem::GetFileItem(sFilePathRel);
-
-	if (!fi)
-		return false;
-
-	fistate = fi->Setup(false);
-	if(fistate == FIST_COMPLETE)
-	{
-		fi->Unreg();
-		return true;
-	}
-
-	SendFmt()<<"Downloading "<<sFilePathRel<<"...\n";
-
-	string sErr;
-	if (RefetchFile(fi, sFilePathRel, sErr))
-	{
-		int code=fi->GetHeader().getStatus();
-		if(301==code || 302==code)
-		{
-			RecDownload(fi, fi->GetHeader().h[header::LOCATION]);
-			code=fi->GetHeader().getStatus();
-		}
-		if(200==code)
-		{
-			bSuccess=true;
-			SendFmt() << "<i>(" << fi->GetTransferCount() / 1024 << "KiB)</i>\n";
-		}
-	}
-
-	if (fi)
-		fi->Unreg();
-
-	SendChunk("<br>\n");
-	return bSuccess;
-}
-
-
 static const string relKey("/Release"), inRelKey("/InRelease");
 
 #define ERRMSGABORT if(m_nErrorCount && m_bErrAbort) { SendChunk(sErr); return; }
@@ -590,7 +606,7 @@ bool tCacheMan::PatchFile(const string &srcRel,
 	{
 		string pfile(diffIdxPathRel.substr(0, diffIdxPathRel.size()-diffIdxSfx.size()+6)
 				+pit->patchName+".gz");
-		if(!DownloadSimple(pfile))
+		if(!Download(pfile, false, VERB_SHOW))
 		{
 			m_indexFilesRel.erase(pfile); // remove the mess for sure
 			SendFmt() << "Failed to download patch file " << pfile << " , stop patching...<br>";
@@ -643,9 +659,7 @@ bool tCacheMan::PatchFile(const string &srcRel,
 	cmd << "cd '" << CACHE_BASE << "_actmp' && red patch.base < combined.diff";
 	if (::system(cmd.c_str()))
 	{
-#ifdef DEBUG
-		SendFmt() << "Command failed: " << cmd << "<br>\n";
-#endif
+		MTLOGASSERT(false, "Command failed: " << cmd);
 		return false;
 	}
 
@@ -653,17 +667,13 @@ bool tCacheMan::PatchFile(const string &srcRel,
 	string respathAbs = CACHE_BASE + sPatchResRel;
 	if (!probe.ScanFile(respathAbs, CSTYPE_SHA1, false))
 	{
-#ifdef DEBUG
-		SendFmt() << "Scan of " << respathAbs << " failed<br>\n";
-#endif
+		MTLOGASSERT(false, "Scan failed: " << respathAbs);
 		return false;
 	}
 
 	if(verifData && probe != *verifData)
 	{
-#ifdef DEBUG
-		SendFmt() << "Verification against: " << respathAbs << " failed<br>\n";
-#endif
+		MTLOGASSERT(false,"Verification failed, against: " << respathAbs);
 		return false;
 	}
 
@@ -671,49 +681,45 @@ bool tCacheMan::PatchFile(const string &srcRel,
 
 }
 
-bool tCacheMan::GetAndCheckHead(cmstring & sDataFileRel, cmstring &sReferencePathRel,
+bool tCacheMan::GetAndCheckHead(cmstring & sTempDataRel, cmstring &sReferencePathRel,
 		off_t nWantedSize)
 {
-	time_t timeStarted=time(0);
-	class tHeadOnlyStorage : public fileitem
+	class fakeCon : public tGetItemDelegate
 	{
 	public:
-		tHeadOnlyStorage(cmstring &path) : fileitem(path)
+		cmstring & sTempDataRel, &sReferencePathRel;
+		off_t nGotSize;
+		fakeCon(cmstring & a, cmstring &b) : sTempDataRel(a), sReferencePathRel(b) {};
+		operator tFileItemPtr()
 		{
-			m_bAllowStoreData=false;
-			m_bHeadOnly=true;
+			class tHeadOnlyStorage : public fileitem
+			{
+			public:
+				fakeCon &m_parent;
+				tHeadOnlyStorage(fakeCon &p)
+				: fileitem(p.sTempDataRel) // storage ref to physical data file
+				, m_parent(p)
+				{
+					m_bAllowStoreData=false;
+					m_bHeadOnly=true;
+					m_head.LoadFromFile(SABSPATH(p.sReferencePathRel+".head"));
+				}
+				void Unreg()
+				{
+					m_head.StoreToFile(m_sPathAbs+".head");
+					m_parent.nGotSize = atoofft(m_head.h[header::CONTENT_LENGTH], -17);
+				};
+			};
+			return tFileItemPtr(static_cast<fileitem*>(new tHeadOnlyStorage(*this)));
 		}
-		int AddHeadData(cmstring &path)
-		{
-			return m_head.LoadFromFile(path+".head");
-		}
-		void ForceStoreHead()
-		{
-			m_head.StoreToFile(m_sPath+".head");
-		}
-	};
-	tHeadOnlyStorage *p=new tHeadOnlyStorage(sDataFileRel);
-	tFileItemPtr pItem(static_cast<fileitem*>(p));
-	string sErr;
-	if(!pItem
-			|| p->AddHeadData(SABSPATH(sReferencePathRel))<=0
-			|| !RefetchFile(pItem, sReferencePathRel, sErr))
-		return false;
-	p->ForceStoreHead();
-
-	const char *plen=pItem->GetHeaderUnlocked().h[header::CONTENT_LENGTH];
-	cmstring sHeadPath=SABSPATH(sDataFileRel)+".head";
-	struct stat stbuf;
-	if(plen && nWantedSize == atoofft(plen)
-			&& 0==stat(sHeadPath.c_str(), &stbuf)
-			)
-	{
-		return timeStarted<=stbuf.st_mtime; // looks sane, fetched right now -> thrust the data
-	}
-	return false;
+	} contor(sTempDataRel, sReferencePathRel);
+	return Download(sReferencePathRel, true, VERB_QUIET, &contor) && contor.nGotSize == nWantedSize;
 }
 
-bool tCacheMan::Inject(cmstring &from, cmstring &to)
+
+
+bool tCacheMan::Inject(cmstring &from, cmstring &to,
+		bool bSetIfileFlags, bool bUpdateRefdata, const header *pHead, bool bTryLink)
 {
 	LOGSTART("tCacheMan::Inject");
 
@@ -722,48 +728,105 @@ bool tCacheMan::Inject(cmstring &from, cmstring &to)
 		return true;
 
 #ifdef DEBUG
-	bool nix = stmiss!=from.find("debian/dists/lenny/contrib/binary-amd64/Packages");
+	bool nix = stmiss!=from.find("debrep/dists/squeeze/non-free/binary-amd64/");
 	SendFmt()<<"Replacing "<<to<<" with " << from <<  "<br>\n";
 #endif
 
-	header h;
-	filereader data;
+	header head;
 
-	if(h.LoadFromFile(SABSPATH(from+".head")) <= 0
-			|| !data.OpenFile(SABSPATH(from), true)
-			|| ! h.h[header::CONTENT_LENGTH]
-			|| off_t(data.GetSize()) != atoofft(h.h[header::CONTENT_LENGTH]))
+	if (!pHead)
 	{
-#ifdef DEBUG
-	SendFmt()<<"Cannot read "<<from<<".head or bad data<br>\n";
-#endif
-		return false;
+		pHead = &head;
+
+		if (head.LoadFromFile(SABSPATH(from+".head")) <= 0 || !head.h[header::CONTENT_LENGTH])
+		{
+			MTLOGASSERT(0, "Cannot read " << from << ".head or bad data");
+			return false;
+		}
+		if(GetFileSize(SABSPATH(from), -1) != atoofft(head.h[header::CONTENT_LENGTH]))
+		{
+			MTLOGASSERT(0, "Bad file size");
+			return false;
+		}
 	}
 
-	tFileItemPtr fi=fileitem::GetFileItem(to);
-	if (!fi)
+	class tInjectItem : public fileitem
+	{
+	public:
+		bool m_link;
+		tInjectItem(cmstring &to, bool bTryLink) : fileitem(to), m_link(bTryLink)
+		{
+		}
+		// noone else should attempt to store file through it
+		virtual bool DownloadStartedStoreHeader(const header &, const char *)
+		{
+			return false;
+		}
+		virtual bool Inject(cmstring &fromRel, const header &head)
+		{
+			m_head = head;
+			if(head.StoreToFile(m_sPathAbs+".head")<=0)
+				return false;
+
+			if(m_link)
+			{
+				setLockGuard;
+				if (LinkOrCopy(SABSPATH(fromRel), m_sPathAbs))
+				{
+					m_status = FIST_COMPLETE;
+					notifyAll();
+					return true;
+				}
+			}
+
+			// shit. Permission problem? Use the old fashioned way :-(
+			filereader data;
+			if(!data.OpenFile(SABSPATH(fromRel), true))
+				return false;
+
+			/* evil...
+			if(!m_link)
+			{
+				// just in case it's the same file, kill the target
+				::unlink(m_sPathAbs.c_str());
+				::unlink((m_sPathAbs+".head").c_str());
+			}
+			*/
+
+			if (Setup(true) >= FIST_LOCALERROR)
+				return false;
+			if (!fileitem::DownloadStartedStoreHeader(head, NULL))
+				return false;
+			if(!StoreFileData(data.GetBuffer(), data.GetSize()) || ! StoreFileData(NULL, 0))
+				return false;
+			if(GetStatus() != FIST_COMPLETE)
+				return false;
+			return true;
+		}
+	};
+	tInjectItem *p=new tInjectItem(to, bTryLink);
+	tFileItemPtr fi(static_cast<fileitem*>(p));
+	if (!fi || !fileitem::RegisterFileItem(to, fi) )
+	{
+		MTLOGASSERT(false, "Couldn't register copy item");
 		return false;
+	}
+	bool bOK = p->Inject(from, *pHead);
 
-	bool bError = ( fi->Setup(true) == FIST_ERROR
-			|| ! fi->DownloadStartedStoreHeader(h, NULL)
-			|| ! fi->StoreFileData(data.GetBuffer(), data.GetSize())
-			|| ! fi->StoreFileData(NULL, 0)
-			|| fi->GetStatus() != FIST_COMPLETE);
-
-	UpdateFingerprint(to, -1, NULL, NULL);
+	if(bOK && bUpdateRefdata)
+		UpdateFingerprint(to, -1, NULL, NULL);
 
 	fi->Unreg();
 
-#ifdef DEBUG
-	if(bError)
+	MTLOGASSERT(bOK, "Inject: failed");
+
+	if(bSetIfileFlags)
 	{
-		LOG("Houston, we have a problem");
-		SendFmt() << "<font color=red>Inject failed</font><br>";
+		tIfileAttribs &atts = SetFlags(to);
+		atts.uptodate = atts.vfile_ondisk = bOK;
 	}
-#endif
-	tIfileAttribs &atts = SetFlags(to);
-	atts.uptodate=atts.vfile_ondisk=!bError;
-	return !bError;
+
+	return bOK;
 }
 
 
@@ -782,7 +845,7 @@ bool tCacheMan::Propagate(cmstring &donorRel, tContId2eqClass::iterator eqClassI
 	struct keeper
 	{
 		tFileItemPtr pItem;
-		keeper(const string &path, bool y) { if(y) pItem=fileitem::GetFileItem(path); }
+		keeper(const string &path, bool y) { if(y) pItem=fileitem::GetRegisteredFileItem(path); }
 		~keeper() { if(pItem) pItem->Unreg(); }
 	} src(donorRel, GetFlags(donorRel).uptodate);
 
@@ -822,10 +885,8 @@ bool tCacheMan::Propagate(cmstring &donorRel, tContId2eqClass::iterator eqClassI
 			// counts fresh file as injected, no need to recheck them in Inject()
 			if (flags.uptodate || Inject(donorRel, tgtCand))
 				nInjCount++;
-#ifdef DEBUG
 			else
-				SendFmt() << "Inject failed<br>\n";
-#endif
+				MTLOGASSERT(false, "Inject failed");
 		}
 	}
 
@@ -847,9 +908,8 @@ bool tCacheMan::Propagate(cmstring &donorRel, tContId2eqClass::iterator eqClassI
 			if(kv!=m_indexFilesRel.end() && kv->second.vfile_ondisk)
 			{
 				kv->second.parseignore=true; // gotcha
-#ifdef DEBUG
-				SendFmt() << "Defused bro of " << *it << ": " << sBro<<"<br>\n";
-#endif
+				MTLOGDEBUG("Defused bro of " << *it << ": " << sBro);
+
 				// also, we don't care if the pdiff index vanished for some reason
 				kv = m_indexFilesRel.find(sBasename+".diff/Index");
 				if(kv!=m_indexFilesRel.end())
@@ -866,10 +926,8 @@ bool tCacheMan::Propagate(cmstring &donorRel, tContId2eqClass::iterator eqClassI
 							&& (broModTime = stbuf.st_mtime, 0 == ::stat(SZABSPATH(donorRel), &stbuf))
 							&& broModTime < stbuf.st_mtime)
 					{
-#ifdef DEBUG
-						SendFmt() << "Unpacked version in " << sBro << " too old, replace from "
-								<< *psTmpUnpackedAbs << "<br>\n";
-#endif
+						MTLOGDEBUG("Unpacked version in " << sBro << " too old, replace from "
+								<< *psTmpUnpackedAbs);
 						FileCopy(*psTmpUnpackedAbs, SZABSPATH(sBro));
 					}
 				}
@@ -922,7 +980,7 @@ void tCacheMan::UpdateIndexFiles()
 		for (tS2IDX::const_iterator it = m_indexFilesRel.begin(); it != m_indexFilesRel.end(); it++)
 		{
 			// nope... tell the fileitem to ignore file data instead ::truncate(SZABSPATH(it->first), 0);
-			if (!DownloadIdxFile(it->first, sErr))
+			if (!Download(it->first, true, VERB_SHOW))
 				m_nErrorCount+=!m_indexFilesRel[it->first].forgiveDlErrors;
 		}
 		ERRMSGABORT;
@@ -931,9 +989,7 @@ void tCacheMan::UpdateIndexFiles()
 
 	typedef map<string, tContId> tFile2Cid;
 
-#ifdef DEBUG
-	SendChunk("<br><br><b>STARTING ULTIMATE INTELLIGENCE</b><br><br>");
-#endif
+	MTLOGDEBUG("<br><br><b>STARTING ULTIMATE INTELLIGENCE</b><br><br>");
 
 	/*
 	 * Update all Release files
@@ -966,11 +1022,10 @@ void tCacheMan::UpdateIndexFiles()
 		if(!endsWith(sPathRel, relKey) && !endsWith(sPathRel, inRelKey))
 			continue;
 
-		if(!DownloadIdxFile(sPathRel, sErr))
+		if(!Download(sPathRel, true, m_indexFilesRel[sPathRel].hideDlErrors
+				? VERB_SHOW_NOERRORS : VERB_SHOW))
 		{
 			m_nErrorCount+=(!m_indexFilesRel[sPathRel].hideDlErrors);
-			if(!m_indexFilesRel[sPathRel].hideDlErrors)
-				SendFmt() << "<font color=\"red\">" << sErr << "</font><br>\n";
 
 			if(CheckAbortCondition())
 				return;
@@ -1138,7 +1193,7 @@ void tCacheMan::UpdateIndexFiles()
 		if (patchidxFileToUse.empty()) // huh, not found? Then just take the first one
 			patchidxFileToUse = itDiffIdx->second.paths.front();
 
-		if (!DownloadIdxFile(patchidxFileToUse, sErr))
+		if (!Download(patchidxFileToUse, true, VERB_SHOW))
 			continue;
 
 		if(CheckAbortCondition())
@@ -1175,18 +1230,13 @@ void tCacheMan::UpdateIndexFiles()
 			if (GetFlags(*its).vfile_ondisk)
 			{
 				header h;
-				if(h.LoadFromFile(SABSPATH(*its)+ ".head")<=0
-						|| ! h.h[header::CONTENT_LENGTH]
-						|| GetFileSize(SABSPATH(*its), -2) != atoofft(h.h[header::CONTENT_LENGTH]))
+				if(h.LoadFromFile(SABSPATH(*its)+ ".head")<=0  || GetFileSize(SABSPATH(*its), -2)
+						!= atoofft(h.h[header::CONTENT_LENGTH], -3))
 				{
-#ifdef DEBUG
-					SendChunk("########### Header looks suspicious<br>");
-#endif
+					MTLOGDEBUG("########### Header looks suspicious");
 					continue;
 				}
-#ifdef DEBUG
-				SendFmt() << "########### Testing file: " << *its << " as patch base candidate<br>";
-#endif
+				MTLOGDEBUG("########### Testing file: " << *its << " as patch base candidate");
 				if (probe.ScanFile(CACHE_BASE + *its, CSTYPE_SHA1, true, df.p))
 				{
 					df.close(); // write the whole file to disk ASAP!
@@ -1235,14 +1285,12 @@ void tCacheMan::UpdateIndexFiles()
 							if(CheckAbortCondition())
 								return;
 
-							h.set(header::LAST_MODIFIED, "Sat, 26 Apr 1986 01:23:39 GMT+3");
+							h.set(header::LAST_MODIFIED, FAKEDATEMARK);
 							h.set(header::CONTENT_LENGTH, pEndSum->size);
 							if (h.StoreToFile(SABSPATH(sPatchResRel) + ".head") <= 0)
 							{
-	#ifdef DEBUG
-								SendFmt() << "############ Failed to store target header as "
-										<< SABSPATH(sPatchResRel) << ".head<br>";
-	#endif
+								MTLOGDEBUG("############ Failed to store target header as "
+										<< SABSPATH(sPatchResRel) << ".head");
 								continue;
 							}
 
@@ -1289,15 +1337,13 @@ void tCacheMan::UpdateIndexFiles()
 				cmstring &cand=*its;
 				if(!endsWith(cand, *ps))
 					continue;
-				if(DownloadIdxFile(cand, sErr))
+				if(Download(cand, true, GetFlags(cand).hideDlErrors ? VERB_SHOW_NOERRORS : VERB_SHOW))
 				{
 					if(CheckAbortCondition())
 						return;
 					if(Propagate(cand, cid2eqcl)) // all locations are covered?
 						goto CONTINUE_NEXT_GROUP;
 				}
-				else if(! GetFlags(cand).hideDlErrors)
-					SendFmt() << "<font color=\"red\">" << sErr << "</font><br>\n";
 
 				if(CheckAbortCondition())
 					return;
@@ -1310,9 +1356,7 @@ void tCacheMan::UpdateIndexFiles()
 			return;
 	}
 
-#ifdef DEBUG
-	SendChunk("<br><br><b>NOW GET THE REST</b><br><br>");
-#endif
+	MTLOGDEBUG("<br><br><b>NOW GET THE REST</b><br><br>");
 
 	// fetch all remaining stuff
 	for(tS2IDX::citer it=m_indexFilesRel.begin(); it!=m_indexFilesRel.end(); it++)
@@ -1320,11 +1364,9 @@ void tCacheMan::UpdateIndexFiles()
 		if(it->second.uptodate || it->second.parseignore || !it->second.vfile_ondisk)
 			continue;
 		string sErr;
-		if(DownloadIdxFile(it->first, sErr))
+		if(Download(it->first, true, it->second.hideDlErrors ? VERB_SHOW_NOERRORS : VERB_SHOW))
 			continue;
 		m_nErrorCount+=(!it->second.forgiveDlErrors);
-		if(!it->second.hideDlErrors)
-			SendFmt() << "<font color=\"red\">" << sErr << "</font><br>\n";
 	}
 
 }
@@ -1397,10 +1439,14 @@ void tCacheMan::InstallBz2edPatchResult(tContId2eqClass::iterator eqClassIter)
 	}
 }
 
-tCacheMan::enumIndexType tCacheMan::GuessIndexType(const mstring &sPath)
+tCacheMan::enumIndexType tCacheMan::GuessIndexTypeFromURL(const mstring &sPath)
 {
 	tStrPos pos = sPath.rfind(SZPATHSEP);
 	string sPureIfileName = (stmiss == pos) ? sPath : sPath.substr(pos + 1);
+
+	if(pos == 0 || sPath.rfind(SZPATHSEP, pos-1)==stmiss) // there should be a valid host part
+		return EIDX_UNSUPPORTED;
+
 	stripSuffix(sPureIfileName, ".gz");
 	stripSuffix(sPureIfileName, ".bz2");
 	stripSuffix(sPureIfileName, ".xz");
@@ -1440,6 +1486,7 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 	bool bNix=StrHas(sPath, "/i18n/");
 #endif
 
+	m_processedIfile = sPath;
 
 	// pre calc relative base folders for later
 	string sCurFilesReferenceDirRel(SZPATHSEP);
@@ -1480,7 +1527,7 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 	reader.AddEofLines();
 
 	// some common variables
-	string sLine, key, val;
+	mstring sLine, key, val;
 	tRemoteFileInfo info;
 	info.SetInvalid();
 	tStrVec vsMetrics;
@@ -1634,7 +1681,7 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 			if(CheckAbortCondition())
 				return true;
 
-			for(tSplitWalk split(sLine, "\"'><=/"); split.Next(); )
+			for(tSplitWalk split(&sLine, "\"'><=/"); split.Next(); )
 			{
 				cmstring tok(split);
 				LOG("testing filename: " << tok);
@@ -1655,7 +1702,7 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 			if(CheckAbortCondition())
 				return true;
 
-			for(tSplitWalk split(sLine, "\"'><=/"); split.Next(); )
+			for(tSplitWalk split(&sLine, "\"'><=/"); split.Next(); )
 			{
 				cmstring tok(split);
 				LOG("testing filename: " << tok);
@@ -1729,7 +1776,7 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 				for(tStrDeq::iterator it=fileList.begin(); it!=fileList.end(); ++it)
 				{
 				// ok, read "checksum size filename" into info and check the word count
-				tSplitWalk split(*it);
+				tSplitWalk split(&*it);
 				if(!split.Next()
 						|| (key = split, !split.Next())
 						|| (val = split, !split.Next())
@@ -1797,7 +1844,7 @@ void tCacheMan::_ProcessSeenIndexFiles()
 		const tIfileAttribs &att=it->second;
 		enumIndexType itype = att.eIdxType;
 		if(!itype)
-			itype=GuessIndexType(it->first);
+			itype=GuessIndexTypeFromURL(it->first);
 		if(!itype) // still unknown. Where does it come from? Just ignore.
 			continue;
 		if(att.parseignore || (!att.vfile_ondisk && !att.uptodate))
@@ -1827,8 +1874,13 @@ void tCacheMan::_ProcessSeenIndexFiles()
 		{
 			if(!m_indexFilesRel[it->first].forgiveDlErrors)
 			{
-				SendChunk("<font color=red>An error occured while reading this file, some contents may have been ignored.</font><br>\n");
 				m_nErrorCount++;
+				SendChunk("<font color=red>An error occured while reading this file, some contents may have been ignored.</font>\n");
+
+				SendFmt() << "<label><input type=\"checkbox\" name=\"kf"
+						<< (nKillLfd++) << "\" value=\"" << it->first
+						<< "\">Tag</label><br>\n";
+				m_bShowControls=true;
 			}
 			continue;
 		}
@@ -1836,9 +1888,7 @@ void tCacheMan::_ProcessSeenIndexFiles()
 		{
 			for(tStrDeq::const_iterator broIt=att.bros->begin(); broIt!=att.bros->end(); broIt++)
 			{
-#ifdef DEBUG
-				SendFmt() << "Marking " << *broIt << " as processed<br>";
-#endif
+				MTLOGDEBUG("Marking " << *broIt << " as processed");
 				SetFlags(*broIt).alreadyparsed=true;
 			}
 		}
@@ -1864,3 +1914,33 @@ void tCacheMan::SetCommonUserFlags(cmstring &cmd)
 	m_bSkipHeaderChecks=(cmd.find("skipHeadChecks")!=stmiss);
 	m_bTruncateDamaged=(cmd.find("truncNow")!=stmiss);
 }
+
+
+void tCacheMan::PrintStats(cmstring &title)
+{
+	multimap<off_t, string> sorted;
+	off_t total=0;
+	for(tS2IDX::const_iterator it=m_indexFilesRel.begin(); it!=m_indexFilesRel.end(); ++it)
+	{
+		if(it->second.space)
+		{
+			sorted.insert(make_pair(it->second.space, it->first));
+			total += it->second.space;
+		}
+	}
+	if(!total)
+		return;
+	if(!m_bVerbose)
+		while(sorted.size() > 10)
+			sorted.erase(sorted.begin());
+	SendFmt() << "<br>\n<table><tr><td colspan=2><u>" << title;
+	if(!m_bVerbose)
+		SendFmt() << " (Top " << sorted.size() << ")";
+	SendFmt() << "</u></td></tr>\n";
+	for(multimap<off_t, string>::reverse_iterator it=sorted.rbegin(); it!=sorted.rend(); ++it)
+	{
+		SendFmt() << "<tr><td><b>" << offttosH(it->first) << "</b></td><td>"
+				<< it->second << "</td></tr>\n";
+	}
+	SendFmt() << "</table>";
+}
