@@ -34,7 +34,7 @@ struct tDlJob
 	mstring sErrorMsg;
 	inline bool HasBrokenStorage()
 	{
-		return (!m_pStorage || m_pStorage->GetStatus() >= FIST_ERROR);
+		return (!m_pStorage || m_pStorage->GetStatus() >= FIST_LOCALERROR);
 	}
 
 #define EFLAG_MORE 0
@@ -396,7 +396,7 @@ struct tDlJob
 				}
 				if (!(e[2] == '\r' && e[3] == '\n'))
 				{
-					aclog::err(m_pStorage->m_sKey + " -- error in chunk format detected");
+					aclog::err(m_pStorage->m_sPathRel + " -- error in chunk format detected");
 					return EFLAG_JOB_BROKEN;
 				}
 
