@@ -29,6 +29,9 @@ using namespace MYSTD;
 #define ERRMSGABORT if(m_nErrorCount && m_bErrAbort) { SendChunk(sErr); return; }
 #define ERRABORT if(m_nErrorCount && m_bErrAbort) { return; }
 
+#define TIMEEXPIRED(t) (t < (m_gMaintTimeNow-acfg::extreshhold*86400))
+#define TIME_AGONY (m_gMaintTimeNow-acfg::extreshhold*86400)
+
 void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsumming)
 {
 	LOGSTART2("expiration::_HandlePkgEntry:",
@@ -89,50 +92,34 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsu
 
 			if (0<h.LoadFromFile(sPathAbs+".head"))
 			{
-				if (h.h[header::CONTENT_LENGTH])
+
+#if 0 // too easy - there should be no inconsistencies here. If that happens, someone might report it.
+				if(h.h[header::LAST_MODIFIED]
+				       && 0==strcmp(h.h[header::LAST_MODIFIED], FAKEDATEMARK))
 				{
-					off_t len=atoofft(h.h[header::CONTENT_LENGTH]);
-					struct stat stinfo;
-					off_t lenInfo=0;
-
-
-#warning XXX RLLY? Review and maybe enable later, should work without it ATM
-					/*
-					// avoid duplicate stat call if the data from the includer is still there
-					if(fprFile.bUnpack == false
-							&& fprFile.csType == CSTYPE_INVALID
-							&& fprFile.size>0)
-					{
-						//SendChunk("data from fingerprint");
-						lenInfo=fprFile.size;
-					}
-
-					else
-					*/if (0==stat(sPathAbs.c_str(), &stinfo))
-					{
-						//SendChunk("data from stat info");
-						lenInfo = stinfo.st_size;
-					}
-					else
-					{
-						SendFmt() << ": error reading attributes, ignoring" << sPathRel;
-						goto keep_in_trash;
-					}
-
-					if(len<lenInfo)
-					{
-						SendFmt() << "<font color=red>WARNING, header file of "
-								<< sPathRel
-								<< " reported too small file size ("
-								<< len << " vs. " << lenInfo
-								<< "), invalidating</font><br>\n";
-						goto keep_in_trash;
-					}
+					// created by us, don't care
+					goto head_checked;
 				}
-				else
+#endif
+
+				off_t len=atoofft(h.h[header::CONTENT_LENGTH], -2);
+				if(len<0)
 				{
 					SendFmt() << "<font color=red>WARNING, header file of "
-							<< sPathRel << " does not contain content length";
+							<< sPathRel << " does not contain content length</font>";
+					goto keep_in_trash;
+				}
+				off_t lenInfo=GetFileSize(sPathAbs, -3);
+				if(lenInfo<0)
+				{
+					SendFmt() << ": error reading attributes, ignoring" << sPathRel;
+					goto keep_in_trash;
+				}
+				if (len < lenInfo)
+				{
+					SendFmt() << "<font color=red>WARNING, header file of " << sPathRel
+							<< " reported too small file size (" << len << " vs. " << lenInfo
+							<< "), invalidating</font><br>\n";
 					goto keep_in_trash;
 				}
 			}
@@ -194,6 +181,8 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsu
 			m_trashCandSet.erase(it++);
 		}
 
+		SetFlags(m_processedIfile).space+=entry.fpr.size;
+
 		continue;
 
 		keep_in_trash:
@@ -201,12 +190,103 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsu
 	}
 }
 
+struct tVerCompKey
+{
+	mstring prevName, ver, prevArcSufx;
+	tS2DAT::iterator it;
+
+	inline bool Set(tS2DAT::iterator &it)
+	{
+		// only care about the doomed ones
+		//if( ! TIMEEXPIRED(it->second.nLostAt))
+		//	return false;
+
+		cmstring &name=it->first.file;
+		tSplitWalk split(&name, "_");
+		if(!split.Next())
+			return false;
+		prevName=split;
+		if(!split.Next())
+			return false;
+		ver=split;
+		for (const char *p = prevArcSufx.c_str(); *p; ++p)
+			if (!isalnum(UINT(*p)) && !strchr(".-+:~", UINT(*p)))
+				return false;
+		if(!split.Next())
+			return false;
+		prevArcSufx=split;
+		this->it=it;
+		return !split.Next(); // no trailing crap
+	}
+	inline bool SamePkg(tVerCompKey &other) const
+	{
+		return other.prevName == prevName && other.prevArcSufx == prevArcSufx;
+	}
+	inline bool operator<(const tVerCompKey &other) const
+	{
+		return CompDebVerLessThan(ver, other.ver);
+	}
+};
+
+inline void expiration::DropExceptionalVersions()
+{
+    if(m_trashCandSet.empty() || !acfg::keepnver)
+    	return;
+
+	int ec=0;
+	MYSTD::set<tVerCompKey> pkgGroup;
+
+	for (tS2DAT::iterator it = m_trashCandSet.begin();; ++it)
+	{
+		tVerCompKey key;
+		if (it != m_trashCandSet.end())
+		{
+			if( ! TIMEEXPIRED(it->second.nLostAt))
+				continue; // already has a life assurance
+			if (!key.Set(it))
+				continue;
+			if (pkgGroup.empty() || pkgGroup.begin()->SamePkg(key))
+			{
+				pkgGroup.insert(key);
+				continue;
+			}
+		}
+
+		// not same pkg or finishing, cleaning the group
+		// no need for tree rotations since count is known
+		int nc = int(pkgGroup.size());
+		if(acfg::keepnver<nc)
+			nc-=acfg::keepnver;
+
+		for (MYSTD::set<tVerCompKey>::iterator lit = pkgGroup.begin();
+				lit != pkgGroup.end() && nc-- > 0; ++lit)
+		{
+			// m_trashCandSet.erase(lit->it);
+			// no point in rotating the tree, set it to oldest non-expirable timestamp
+			lit->it->second.nLostAt = TIME_AGONY;
+			ec++;
+		}
+
+		if (it == m_trashCandSet.end())
+			break;
+
+		// restart the group
+		pkgGroup.clear();
+		pkgGroup.insert(key);
+	}
+	if(ec)
+		SendFmt() << "Kept " << ec << " more package(s) not covered by index references<br>\n";
+}
+
 inline void expiration::RemoveAndStoreStatus(bool bPurgeNow)
 {
 	LOGSTART("expiration::_RemoveAndStoreStatus");
 	FILE *f(NULL);
     if(!bPurgeNow)
     {
+
+        DropExceptionalVersions();
+
         string sDbFileAbs=CACHE_BASE+"_expending_dat";
 
         f = fopen(sDbFileAbs.c_str(), "w");
@@ -258,12 +338,11 @@ inline void expiration::RemoveAndStoreStatus(bool bPurgeNow)
 
 	    //cout << "Took " << sWhatHead << " from the list" <<endl;
 
-		if(bPurgeNow || (it->second.nLostAt < (m_nTimeNow-acfg::extreshhold*86400)))
+		if(bPurgeNow || TIMEEXPIRED(it->second.nLostAt))
 		{
 			SendFmt() << "Removing " << sPathRel << "<br>\n";
 
 #ifdef ENABLED
-
 			::unlink(sPathAbs.c_str());
 			::unlink((sPathAbs+".head").c_str());
 			::rmdir((CACHE_BASE + k.dirRel).c_str());
@@ -271,7 +350,10 @@ inline void expiration::RemoveAndStoreStatus(bool bPurgeNow)
         }
 		else if(f)
 		{
-			SendFmt() << "Tagging " << sPathRel << "<br>\n";
+			SendFmt() << "Tagging " << sPathRel;
+			if(m_bVerbose)
+				SendFmt() << " (t-"<<(m_gMaintTimeNow-it->second.nLostAt)/3600 << "h)";
+			SendChunk("<br>\n");
 
 			n++;
 			tagSpace+=it->second.fpr.size;
@@ -377,7 +459,8 @@ void expiration::Action(const string & cmd)
 
 	if (m_bErrAbort && m_nErrorCount>0)
 	{
-		SendChunk("<font color=\"red\">Found errors during processing, aborting as requested.</font>"
+		SendChunk("<font color=\"red\">Found errors during processing, "
+				"aborting as requested.</font>"
 			"<!-- TELL:THE:ADMIN -->");
 		return;
 	}
@@ -393,7 +476,9 @@ void expiration::Action(const string & cmd)
 
 	DelTree(CACHE_BASE+"_actmp");
 
-	SendChunk("Done.");
+	PrintStats("Assigned disk space");
+
+	SendChunk("<br>Done.</br>");
 
 }
 
@@ -464,12 +549,7 @@ bool expiration::ProcessRegular(const string & sPathAbs, const struct stat &stin
 	if(sPathAbs.size()<=CACHE_BASE_LEN) // heh?
 		return false;
 
-	if (++m_nProgIdx == m_nProgTell)
-	{
-		SendFmt()<<"Scanning, found "<<m_nProgIdx<<" file"
-				<< (m_nProgIdx>1?"s":"") << "...<br />\n";
-		m_nProgTell*=2;
-	}
+	ProgTell();
 
 	string sPathRel(sPathAbs, CACHE_BASE_LEN);
 
@@ -482,10 +562,10 @@ bool expiration::ProcessRegular(const string & sPathAbs, const struct stat &stin
 	else
 	{
 		AddIFileCandidate(sPathRel);
-#warning for d-i images, also add their MD5SUMS file, always
+#warning TODO: when Lenny is history, add additional MD5SUMS file to the list for d-i image files
 		//SendChunk(string("<br>hm?<br>")+sDirnameAndSlash + " -- " + sBasename);
 		tDiskFileInfo &finfo = m_trashCandSet[sPathRel];
-		finfo.nLostAt = m_nTimeNow;
+		finfo.nLostAt = m_gMaintTimeNow;
 		finfo.fpr.size = stinfo.st_size;
 	}
     return true;
@@ -536,7 +616,7 @@ void expiration::LoadTrashMapFromFile(bool bForceInsert)
 		char *eptr(NULL);
 		const char *s = sLine.c_str();
 		time_t timestamp = strtoull(s, &eptr, 10);
-		if (!eptr || *eptr != '\t' || !timestamp || timestamp > m_nTimeNow) // where is the DeLorean?
+		if (!eptr || *eptr != '\t' || !timestamp || timestamp > m_gMaintTimeNow) // where is the DeLorean?
 			continue;
 		const char *sep = strchr(++eptr, (unsigned) '\t');
 		if (!sep)
