@@ -36,45 +36,54 @@ fileitem::fileitem(MYSTD::string sPath) :
 	condition(),
 	m_nIncommingCount(0),
 	m_nSizeSeen(0),
-	m_sPath(acfg::cachedir+SZPATHSEP+sPath),
-	m_sKey(sPath),
 	m_bCheckFreshness(true),
 	m_bHeadOnly(false),
 	m_bAllowStoreData(true),
 	m_nSizeChecked(0),
 	m_filefd(-1),
 	m_nDlRefsCount(0),
-	status(FIST_FRESH),
+	m_status(FIST_FRESH),
 	m_nTimeExpireAt(0)
 {
 
-	// not using hexmap, just converting only few allowed chars
-
-	for (int i = 0; i < int(m_sPath.length()) - 3; i++)
+	// FIXME: this sucks, belongs into the fileitem
+	if(acfg::stupidfs)
 	{
-		if (m_sPath[i] == '%')
+		// fix weird chars that we don't like in the filesystem
+		replaceChars(sPath, ENEMIESOFDOSFS, '_');
+		replaceChars(sPath, ENEMIESOFDOSFS, '_');
+#ifdef WIN32
+		replaceChars(sPath, "/", '\\');
+#endif
+	}
+
+	// not using hexmap yet, convert just what is really needed for APT
+
+	for (int i = 0; i < int(sPath.length()) - 3; i++)
+	{
+		if (sPath[i] == '%')
 		{
-			switch (m_sPath[i + 1])
+			switch (sPath[i + 1])
 			{
 			case '7':
-				switch (m_sPath[i + 2])
+				switch (sPath[i + 2])
 				{
 				case 'E':
 				case 'e':
-					m_sPath.replace(i, 3, "~", 1);
+					sPath.replace(i, 3, "~", 1);
 					break;
 				}
 				break;
 			case '5':
-				switch (m_sPath[i + 2])
+				switch (sPath[i + 2])
 				{
 				case 'B':
 				case 'b':
-					m_sPath.replace(i, 3, "[", 1);
+					sPath.replace(i, 3, "[", 1);
 					break;
 				case 'D':
 				case 'd':
-					m_sPath.replace(i, 3, "]", 1);
+					sPath.replace(i, 3, "]", 1);
 					break;
 				}
 				break;
@@ -83,6 +92,10 @@ fileitem::fileitem(MYSTD::string sPath) :
 			}
 		}
 	}
+
+	m_sPathRel = sPath;
+	m_sPathAbs = acfg::cacheDirSlash+sPath;
+
 	//ldbg("item created, m_sPath: "<< m_sPath);
 }
 
@@ -104,15 +117,15 @@ void fileitem::DecDlRefCount(const string &sReason)
 	
 	// ... otherwise: the last downloader disappeared, need to tell observers
 
-	if (status<FIST_COMPLETE)
+	if (m_status<FIST_COMPLETE)
 	{
-		status=FIST_ERROR;
+		m_status=FIST_LOCALERROR;
 		m_head.clear();
 		m_head.frontLine=string("HTTP/1.1 ")+sReason;
 		m_head.type=header::ANSWER;
 
 		if (acfg::debug>1)
-			aclog::misc(string("Download of ")+m_sKey+" aborted");
+			aclog::misc(string("Download of ")+m_sPathRel+" aborted");
 	}
 	checkforceclose(m_filefd);
 }
@@ -128,14 +141,14 @@ uint64_t fileitem::GetTransferCount()
 int fileitem::GetFileFd() {
 	LOGSTART("fileitem::GetFileFd");
 	setLockGuard;
-	if(status<FIST_DLGOTHEAD)
+	if(m_status<FIST_DLGOTHEAD)
 		return -1;
-	ldbg("Opening " << m_sPath);
-	int fd=open(m_sPath.c_str(), O_RDONLY);
+	ldbg("Opening " << m_sPathAbs);
+	int fd=open(m_sPathAbs.c_str(), O_RDONLY);
 	
 #ifdef HAVE_FADVISE
 	// optional, experimental
-	if(status==FIST_COMPLETE)
+	if(m_status==FIST_COMPLETE)
 		posix_fadvise(fd, 0, m_nSizeChecked, POSIX_FADV_SEQUENTIAL);
 #endif
 
@@ -154,7 +167,7 @@ void fileitem::ResetCacheState()
 	setLockGuard;
 	m_nSizeSeen = 0;
 	m_nSizeChecked = 0;
-	status = FIST_FRESH;
+	m_status = FIST_FRESH;
 	m_bAllowStoreData = true;
 	m_head.clear();
 }
@@ -165,19 +178,19 @@ FiStatus fileitem::Setup(bool bCheckFreshness)
 
 	setLockGuard;
 
-	if(status>FIST_FRESH)
-		return status;
-	status=FIST_INITED;
+	if(m_status>FIST_FRESH)
+		return m_status;
+	m_status=FIST_INITED;
 	m_bCheckFreshness = bCheckFreshness;
 	
-	if(m_head.LoadFromFile(m_sPath+".head") >0 && m_head.type==header::ANSWER )
+	if(m_head.LoadFromFile(m_sPathAbs+".head") >0 && m_head.type==header::ANSWER )
 	{
 		if(200 != m_head.getStatus())
 			goto error_clean;
 		
 		LOG("good head");
 
-		m_nSizeSeen=GetFileSize(m_sPath, 0);
+		m_nSizeSeen=GetFileSize(m_sPathAbs, 0);
 
 		// some plausibility checks
 		if(m_bCheckFreshness)
@@ -204,7 +217,7 @@ FiStatus fileitem::Setup(bool bCheckFreshness)
 				// is it complete? and 0 value also looks weird, try to verify
 				if(m_nSizeSeen == nContLen && nContLen>0)
 				{
-					status=FIST_COMPLETE;
+					m_status=FIST_COMPLETE;
 					m_nSizeChecked=m_nSizeSeen;
 				}
 			}
@@ -220,17 +233,34 @@ FiStatus fileitem::Setup(bool bCheckFreshness)
 		// maybe there is some left-over without head file?
 		// Don't thrust volatile data, but otherwise try to reuse?
 		if(!bCheckFreshness)
-			m_nSizeSeen=GetFileSize(m_sPath, 0);
+			m_nSizeSeen=GetFileSize(m_sPathAbs, 0);
 	}
-	LOG("resulting status: " << status);
-	return status;
+	LOG("resulting status: " << m_status);
+	return m_status;
 
 	error_clean:
-			unlink((m_sPath+".head").c_str());
+			unlink((m_sPathAbs+".head").c_str());
 			m_head.clear();
 			m_nSizeSeen=0;
-			status=FIST_INITED;
-			return status; // unuseable, to be redownloaded
+			m_status=FIST_INITED;
+			return m_status; // unuseable, to be redownloaded
+}
+
+
+bool fileitem::SetupClean()
+{
+	setLockGuard;
+
+	if(m_status>FIST_FRESH)
+		return false;
+	m_status=FIST_INITED;
+
+	unlink(m_sPathAbs.c_str());
+	unlink((m_sPathAbs+".head").c_str());
+	m_head.clear();
+	m_nSizeSeen=m_nSizeChecked=0;
+
+	return true;
 }
 
 inline void _LogWithErrno(const char *msg, const string & sFile)
@@ -245,13 +275,13 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 
 	setLockGuard;
 
-	USRDBG(5, "Download started, storeHeader for " << m_sKey << ", current status: " << status);
+	USRDBG(5, "Download started, storeHeader for " << m_sPathRel << ", current status: " << m_status);
 	
-	if(status > FIST_DLPENDING) // already started? error? whatever
+	if(m_status > FIST_DLPENDING) // already started? error? whatever
 		return false;
 	
-	if(status<FIST_DLGOTHEAD)
-		status=FIST_DLGOTHEAD; // assume that for now, may become error state...
+	if(m_status<FIST_DLGOTHEAD)
+		m_status=FIST_DLGOTHEAD; // assume that for now, may become error state...
 	
 	if(m_bCheckFreshness)
 		m_nTimeExpireAt = time(0) + MAXTEMPDELAY;
@@ -265,13 +295,13 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 	notifyAll();
 
 #define SETERROR(x) { m_bAllowStoreData=false; m_head.frontLine="HTTP/1.1 "; \
-	m_head.frontLine+=x; status=FIST_ERROR; _LogWithErrno(x, m_sPath); }
+	m_head.frontLine+=x; m_status=FIST_LOCALERROR; _LogWithErrno(x, m_sPathRel); }
 #define SETERRORKILLFILE(x) { SETERROR(x); goto kill_file; }
 #define BOUNCE(x) { SETERROR(x); return false; }
 
 	m_head.set(header::XORIG, h.h[header::XORIG]);
 
-	string sHeadPath=m_sPath+".head";
+	string sHeadPath=m_sPathAbs+".head";
 	
 	switch(h.getStatus())
 	{
@@ -279,8 +309,7 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 	{
 		m_nSizeChecked=0;
 		m_head=h;
-		const char *p=h.h[header::CONTENT_LENGTH];
-		hint_length=p ? atoofft(p) : 0;
+		hint_length=atoofft(h.h[header::CONTENT_LENGTH], 0);
 		break;
 	}
 	case 206:
@@ -324,7 +353,7 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 		// special optimization; if "-1 trick" was used then maybe don't reopen that file for writing later
 		if(m_bCheckFreshness && pNextData && m_nSizeSeen == mylen && m_nSizeChecked == mylen-1)
 		{
-			int fd=open(m_sPath.c_str(), O_RDONLY);
+			int fd=open(m_sPathAbs.c_str(), O_RDONLY);
 			if(fd>=0)
 			{
 				if(m_nSizeChecked==lseek(fd, m_nSizeChecked, SEEK_SET))
@@ -333,7 +362,7 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 					if(1 == read(fd, &c, 1) && c == *pNextData)
 					{
 						aclog::err("known data hit, don't write to...");
-						aclog::err(m_sPath);
+						aclog::err(m_sPathAbs);
 						m_bAllowStoreData=false;
 						m_nSizeChecked=mylen;
 					}
@@ -362,7 +391,7 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 		m_head.frontLine=h.frontLine;
 		m_head.type=header::ANSWER;
 		m_head.set(header::CONTENT_LENGTH, "0");
-		if(status>FIST_DLGOTHEAD)
+		if(m_status>FIST_DLGOTHEAD)
 		{
 			// oh shit. Client may have already started sending it. Prevent such trouble in future.
 			unlink(sHeadPath.c_str());
@@ -370,7 +399,7 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 	}
 
 	if(acfg::debug>1)
-		aclog::misc(string("Download of ")+m_sKey+" started");
+		aclog::misc(string("Download of ")+m_sPathRel+" started");
 
 	if(m_bAllowStoreData)
 	{
@@ -383,8 +412,8 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 		int flags = O_WRONLY | O_CREAT | O_BINARY;
 		struct stat stbuf;
 				
-		mkbasedir(m_sPath);
-		m_filefd=open(m_sPath.c_str(), flags, acfg::fileperms);
+		mkbasedir(m_sPathAbs);
+		m_filefd=open(m_sPathAbs.c_str(), flags, acfg::fileperms);
 		ldbg("file opened?! returned: " << m_filefd);
 		
 		// self-recovery from cache poisoned with files with wrong permissions
@@ -392,27 +421,27 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 		{
 			if(m_nSizeChecked>0) // OOOH CRAP! CANNOT APPEND HERE! Do what's still possible.
 			{
-				string temp=m_sPath+".tmp";
-				if(FileCopy(m_sPath, temp) && 0==unlink(m_sPath.c_str()) )
+				string temp=m_sPathAbs+".tmp";
+				if(FileCopy(m_sPathAbs, temp) && 0==unlink(m_sPathAbs.c_str()) )
 				{
-					if(0!=rename(temp.c_str(), m_sPath.c_str()))
+					if(0!=rename(temp.c_str(), m_sPathAbs.c_str()))
 						BOUNCE("503 Cannot rename files");
 					
 					// be sure about that
-					if(0!=stat(m_sPath.c_str(), &stbuf) || stbuf.st_size!=m_nSizeSeen)
+					if(0!=stat(m_sPathAbs.c_str(), &stbuf) || stbuf.st_size!=m_nSizeSeen)
 						BOUNCE("503 Cannot copy file parts, filesystem full?");
 					
-					m_filefd=open(m_sPath.c_str(), flags, acfg::fileperms);
+					m_filefd=open(m_sPathAbs.c_str(), flags, acfg::fileperms);
 					ldbg("file opened after copying around: ");
 				}
 				else
 					BOUNCE((tSS()<<"503 Cannot store or remove files in "
-							<< GetDirPart(m_sPath)).c_str());
+							<< GetDirPart(m_sPathAbs)).c_str());
 			}
 			else
 			{
-				unlink(m_sPath.c_str());
-				m_filefd=open(m_sPath.c_str(), flags, acfg::fileperms);
+				unlink(m_sPathAbs.c_str());
+				m_filefd=open(m_sPathAbs.c_str(), flags, acfg::fileperms);
 				ldbg("file force-opened?! returned: " << m_filefd);
 			}
 		}
@@ -421,7 +450,7 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 		{
 			aclog::errnoFmter efmt("503 Cache storage error - ");
 #ifdef DEBUG
-			BOUNCE((efmt+m_sPath).c_str());
+			BOUNCE((efmt+m_sPathAbs).c_str());
 #else
 			BOUNCE(efmt.c_str());
 #endif
@@ -446,13 +475,6 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 		
 		ldbg("Storing header as "+sHeadPath);
 		int count=m_head.StoreToFile(sHeadPath);
-		
-		// unlink and retry
-		if(count<0)
-		{
-			unlink(sHeadPath.c_str());
-			count=m_head.StoreToFile(sHeadPath);
-		}
 
 		if(count<0)
 			SETERRORKILLFILE( (-count!=ENOSPC ? "503 Cache storage error" : "503 OUT OF DISK SPACE"));
@@ -465,7 +487,7 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 			SETERRORKILLFILE("503 IO error, positioning");
 	}
 	
-	status=FIST_DLGOTHEAD;
+	m_status=FIST_DLGOTHEAD;
 	return true;
 
 	kill_file:
@@ -477,11 +499,11 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 		forceclose(m_filefd);
 	}
 
-	LOG("Deleting " << m_sPath);
-	unlink(m_sPath.c_str());
+	LOG("Deleting " << m_sPathAbs);
+	unlink(m_sPathAbs.c_str());
 	unlink(sHeadPath.c_str());
 	
-	status=FIST_ERROR;
+	m_status=FIST_LOCALERROR;
 	return false;
 }
 
@@ -490,39 +512,39 @@ bool fileitem::StoreFileData(const char *data, unsigned int size)
 
 	setLockGuard;
 
-	LOGSTART2("fileitem::StoreFileData", "status: " << status << ", size: " << size);
+	LOGSTART2("fileitem::StoreFileData", "status: " << m_status << ", size: " << size);
 
 	// something might care, most likely... also about BOUNCE action
 	notifyAll();
 	
 	m_nIncommingCount+=size;
 	
-	if(status >= FIST_ERROR || status < FIST_DLGOTHEAD)
+	if(m_status >= FIST_LOCALERROR || m_status < FIST_DLGOTHEAD)
 		return false;
 	
 	if (size==0)
 	{
-		if(FIST_COMPLETE == status)
+		if(FIST_COMPLETE == m_status)
 		{
 			LOG("already completed");
 			return true;
 		}
 
-		status=FIST_COMPLETE;
+		m_status=FIST_COMPLETE;
 
 		if(acfg::debug>1)
-			aclog::misc(tSS()<<"Download of "<<m_sKey<<" finished");
+			aclog::misc(tSS()<<"Download of "<<m_sPathRel<<" finished");
 		
 		// we are done! Fix header from chunked transfers?
 		if (m_filefd>=0 && ! m_head.h[header::CONTENT_LENGTH])
 		{
 			m_head.set(header::CONTENT_LENGTH, m_nSizeChecked);
-			m_head.StoreToFile(m_sPath+".head");
+			m_head.StoreToFile(m_sPathAbs+".head");
 		}
 	}
 	else
 	{
-		status = FIST_DLRECEIVING;
+		m_status = FIST_DLRECEIVING;
 
 		if (!m_bAllowStoreData)
 			return true;
@@ -573,7 +595,7 @@ void fileitem::Unreg()
 
    lockguard managementLock(mapLck); 
 	
-   std::map<string,tFileRefEntry>::iterator entry = mapItems.find(m_sKey);
+   std::map<string,tFileRefEntry>::iterator entry = mapItems.find(m_sPathRel);
    if(mapItems.end() == entry)
    {
 	   aclog::err("INTERNAL ERROR, attempt to unregister non-existing download item");
@@ -589,13 +611,13 @@ void fileitem::Unreg()
    {
 	   LOG("*this is last entry, deleting dl/fi mapping");
 	  setLockGuard;
-	  status=FIST_ERRNOUSER;
+	  m_status=FIST_ERRNOUSER;
 	  notifyAll();
 	  mapItems.erase(entry);
    }
 }
 
-tFileItemPtr fileitem::GetFileItem(MYSTD::string sPathKey)
+tFileItemPtr fileitem::GetRegisteredFileItem(MYSTD::string sPathKey)
 {
 	LOGSTART2s("fileitem::GetFileItem", sPathKey);
 
@@ -632,6 +654,24 @@ tFileItemPtr fileitem::GetFileItem(MYSTD::string sPathKey)
 	return p;
 }
 
+bool fileitem::RegisterFileItem(mstring sPathRel,
+		tFileItemPtr spCustomFileItem)
+		{
+			if (sPathRel.empty() || !spCustomFileItem)
+				return false;
+
+			lockguard lockGlobalMap(mapLck);
+
+			map<string, tFileRefEntry>::iterator it = mapItems.find(sPathRel);
+			if (it != mapItems.end())
+				return false; // conflict, another agent is already active
+
+			tFileRefEntry & entry = mapItems[sPathRel];
+			entry.ref = spCustomFileItem;
+			entry.nRefCount = 1;
+
+			return true;
+		}
 
 // Opposite of GetFileItem, unregisters existing file item now or as soon as possible.
 void fileitem::UnregOneASAP(tFileItemPtr pItem)
@@ -645,7 +685,7 @@ void fileitem::UnregOneASAP(tFileItemPtr pItem)
 		return;
 	}
 	lockguard lockGlobalMap(mapLck);
-	std::map<string, tFileRefEntry>::iterator entry = mapItems.find(pItem->m_sKey);
+	std::map<string, tFileRefEntry>::iterator entry = mapItems.find(pItem->m_sPathRel);
 	// not existing? -> not for us. Still enough references? -> let this one go
 	if (mapItems.end() == entry || entry->second.nRefCount>1)
 	{
@@ -689,7 +729,7 @@ time_t fileitem::DoDelayedUnregAndCheck()
 		}
 		// ok, unused and delay is over. Destroy with the same sequence as Unreg... does.
 		lockguard g(here->second.ref.get());
-		here->second.ref->status=FIST_ERRNOUSER;
+		here->second.ref->m_status=FIST_ERRNOUSER;
 		here->second.ref->notifyAll();
 		g.unLock();
 		mapItems.erase(here);
@@ -712,3 +752,13 @@ void DumpItems()
 	*/
 }
 #endif
+
+FiStatus fileitem::WaitForFinish(int *httpCode)
+{
+	setLockGuard;
+	while(m_status<FIST_COMPLETE)
+		wait();
+	if(httpCode)
+		*httpCode=m_head.getStatus();
+	return m_status;
+}
