@@ -183,11 +183,14 @@ void filereader::Close()
 		m_szFileBuf=(char*)MAP_FAILED;
 	}
 			
+  /*
 	if(m_fd>=0)
 	{
 		fsync(m_fd);
 		forceclose(m_fd);
 	}
+  */
+  checkforceclose(m_fd);
 	
 #ifdef HAVE_LIBBZ2
 	if(flags&FLAG_BZ2)
@@ -198,6 +201,12 @@ void filereader::Close()
 	if(flags&FLAG_GZ)
 		deflateEnd(&m_zStream);
 #endif
+
+#ifdef HAVE_LZMA
+	if(flags&FLAG_LZMA)
+     lzma_end(&m_xzStream);
+#endif
+
 	flags=0;
 	m_nBufSize=0;
 }
