@@ -13,7 +13,7 @@ using namespace MYSTD;
 #include "fileitem.h"
 #include "dlcon.h"
 #include "sockio.h"
-#include "fileio.h" // for ::stat
+#include "fileio.h" // for ::stat and related macros
 #include <dirent.h>
 #include <algorithm>
 #include "maintenance.h"
@@ -24,6 +24,7 @@ using namespace MYSTD;
 
 #include <errno.h>
 
+// XXX no longer needed after debug logging overhaul
 #define REPLEVEL (m_type==rechecks::FILE_INDEX ? 4 : (m_type==rechecks::FILE_PKG ? 5 : SPAMLEVEL))
 
 
@@ -53,16 +54,16 @@ public:
 	virtual FiStatus Setup(bool)
 	{
 		m_nSizeChecked = m_nSizeSeen = 0;
-		return status = FIST_INITED;
+		return m_status = FIST_INITED;
 	}
 	virtual void Unreg() {
 		setLockGuard;
-		if(FIST_DLRECEIVING == status)
+		if(FIST_DLRECEIVING == m_status)
 		{
 			// no shit, there is some input waiting? bring the sender down ASAP
 			// this is not nice to the sender because it triggers a reconnection, but
 			// it's a local source so it shouldn't hurt much
-			status = FIST_ERRNOUSER;
+			m_status = FIST_ERRNOUSER;
 			notifyAll();
 		}
 	};
@@ -71,14 +72,14 @@ public:
 	{
 		setLockGuard;
 		m_head=h;
-		status=FIST_DLGOTHEAD;
+		m_status=FIST_DLGOTHEAD;
 		return true;
 	}
 	virtual bool StoreFileData(const char *data, unsigned int size)
 	{
 		setLockGuard;
 
-		LOGSTART2("tPassThroughFitem::StoreFileData", "status: " << status);
+		LOGSTART2("tPassThroughFitem::StoreFileData", "status: " << m_status);
 
 		// something might care, most likely... also about BOUNCE action
 		notifyAll();
@@ -86,25 +87,25 @@ public:
 		m_nIncommingCount += size;
 
 		dbgline;
-		if (status >= FIST_ERROR || status < FIST_DLGOTHEAD)
+		if (m_status >= FIST_LOCALERROR || m_status < FIST_DLGOTHEAD)
 			return false;
 
 		if (size == 0)
-			status = FIST_COMPLETE;
+			m_status = FIST_COMPLETE;
 		else
 		{
 			dbgline;
-			status = FIST_DLRECEIVING;
+			m_status = FIST_DLRECEIVING;
 			m_nSizeChecked += size;
 			m_pData = data;
 			m_nConsumable=size;
 			m_nConsumed=0;
-			while(0 == m_nConsumed && status<FIST_ERROR)
+			while(0 == m_nConsumed && m_status<FIST_LOCALERROR)
 				wait();
 
 			dbgline;
 			// let the downloader abort?
-			if(status >= FIST_ERROR)
+			if(m_status >= FIST_LOCALERROR)
 				return false;
 
 			dbgline;
@@ -118,12 +119,12 @@ public:
 	{
 		setLockGuard;
 
-		while(0 == m_nConsumable && status<FIST_ERROR
-				&& ! (m_nSizeChecked==0 && status==FIST_COMPLETE))
+		while(0 == m_nConsumable && m_status<FIST_LOCALERROR
+				&& ! (m_nSizeChecked==0 && m_status==FIST_COMPLETE))
 		{
 			wait();
 		}
-		if (status >= FIST_ERROR || !m_pData)
+		if (m_status >= FIST_LOCALERROR || !m_pData)
 			return -1;
 
 		if(!m_nSizeChecked)
@@ -134,7 +135,7 @@ public:
 			r = 0;
 		if(r<0)
 		{
-			status=FIST_ERROR;
+			m_status=FIST_LOCALERROR;
 		}
 		else if(r>0)
 		{
@@ -158,7 +159,7 @@ public:
 
 	tGeneratedFitemBase(const string &sFitemId, const char *szFrontLineMsg) : fileitem(sFitemId), m_data(256)
 	{
-		status=FIST_COMPLETE;
+		m_status=FIST_COMPLETE;
 		m_head.type = header::ANSWER;
 		m_head.frontLine = "HTTP/1.1 ";
 		m_head.frontLine += (szFrontLineMsg ? szFrontLineMsg : "500 Internal Failure");
@@ -166,7 +167,7 @@ public:
 	}
 	ssize_t SendData(int out_fd, int, off_t &nSendPos, size_t nMax2SendNow)
 	{
-		if (status >= FIST_ERROR || out_fd<0)
+		if (m_status >= FIST_LOCALERROR || out_fd<0)
 			return -1;
 		int r = m_data.syswrite(out_fd, nMax2SendNow);
 		if(r>0) nSendPos+=r;
@@ -292,16 +293,6 @@ const string & GetMimeType(const string &path)
 	return nix;
 }
 
-void replaceChars(string &s, const char *szBadChars, char goodChar)
-{
-	for(string::iterator p=s.begin();p!=s.end();p++)
-		for(const char *b=szBadChars;*b;b++)
-			if(*b==*p)
-			{
-				*p=goodChar;
-				break;
-			}
-}
 
 inline void job::HandleLocalDownload(const string &visPath,
 		const string &fsBase, const string &fsSubpath)
@@ -456,8 +447,8 @@ inline void job::HandleLocalDownload(const string &visPath,
 		tLocalGetFitem(string s, string sLocalPath, struct stat &stdata) : fileitem(s)
 		{
 			m_bAllowStoreData=false;
-			m_sPath=sLocalPath;
-			status=FIST_COMPLETE;
+			m_sPathAbs=sLocalPath;
+			m_status=FIST_COMPLETE;
 			m_nSizeChecked=m_nSizeSeen=stdata.st_size;
 			m_bCheckFreshness=false;
 			m_head.type=header::ANSWER;
@@ -483,7 +474,7 @@ void job::PrepareDownload() {
 
     string sRawUriPath, sPathResdiual;
     tHttpUrl tUrl; // parsed URL
-        
+
     const acfg::tRepoData * pBackends(NULL); // appropriate backends
     const string * psVname(NULL); // virtual name for storage pool, if available
     
@@ -538,12 +529,9 @@ void job::PrepareDownload() {
 			goto report_invport;
 		}
 
-		// make a shortcut
-		string & sPath=tUrl.sPath;
-
 		// kill multiple slashes
-		for(tStrPos pos; stmiss != (pos = sPath.find("//")); )
-			sPath.erase(pos, 1);
+		for(tStrPos pos; stmiss != (pos = tUrl.sPath.find("//")); )
+			tUrl.sPath.erase(pos, 1);
 
 		bPtMode=rechecks::MatchUncacheableRequest(tUrl.ToURI());
 
@@ -576,14 +564,14 @@ void job::PrepareDownload() {
 		}
 
 		// entered directory but not defined as local? Then 404 it with hints
-		if(!sPath.empty() && endsWithSzAr(sPath, "/"))
+		if(!tUrl.sPath.empty() && endsWithSzAr(tUrl.sPath, "/"))
 		{
 			LOG("generic user information page");
 			m_sMaintCmd="/";
 			return;
 		}
 
-		m_type = rechecks::GetFiletype(sPath);
+		m_type = rechecks::GetFiletype(tUrl.sPath);
 
 		if ( m_type == FILE_INVALID ) goto report_notallowed;
 		
@@ -597,37 +585,17 @@ void job::PrepareDownload() {
 			m_sFileLoc=*psVname+SZPATHSEP+sPathResdiual;
 		else
 			m_sFileLoc=tUrl.sHost+tUrl.sPath;
-		
-		// FIXME: this sucks, belongs into the fileitem
-		if(acfg::stupidfs)
-		{
-			// fix weird chars that we don't like in the filesystem
-			replaceChars(tUrl.sPath, ENEMIESOFDOSFS, '_');
-			replaceChars(tUrl.sHost, ENEMIESOFDOSFS, '_');
-#ifdef WIN32
-			replaceChars(tUrl.sPath, "/", '\\');
-#endif
-		}
+
+		bForceFreshnessChecks = ( ! acfg::offlinemode && m_type==FILE_INDEX);
+
+    	m_pItem=fileitem::GetRegisteredFileItem(m_sFileLoc);
+
 	}
 	MYCATCH(MYSTD::out_of_range&) // better safe...
 	{
     	goto report_invpath;
     }
     
-    bForceFreshnessChecks = ( ! acfg::offlinemode && m_type==FILE_INDEX);
-
-#ifdef QUATSCH
-    // some users want to not be most up-to-date
-	if(bForceFreshnessChecks && acfg::updinterval > 0)
-	{
-		struct stat stbuf;
-		bForceFreshnessChecks=(0 == ::stat(m_sFileLoc.c_str(), &stbuf)
-				&& ( time(0) - (UINT) stbuf.st_ctime) < acfg::updinterval);
-	}
-#endif
-
-	m_pItem=fileitem::GetFileItem(m_sFileLoc);
-    
     if(!m_pItem)
     {
     	USRDBG(nix, "Error creating file item for " << m_sFileLoc);
@@ -660,7 +628,7 @@ void job::PrepareDownload() {
     	dbgline;
     	MYTRY
 		{
-			if(psVname && NULL != (pBackends=acfg::GetBackendVec(psVname)))
+			if(psVname && NULL != (pBackends=acfg::GetBackendVec(*psVname)))
 			{
 				LOG("Backends found, using them with " << sPathResdiual
 						<< ", first backend: " <<pBackends->m_backends.front().ToURI());
@@ -735,7 +703,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 	}
 	
 	off_t nGooddataSize(0);
-	FiStatus fistate(FIST_ERROR);
+	FiStatus fistate(FIST_LOCALERROR);
 	header respHead; // template of the response header, e.g. as found in cache
 
 	if (confd<0)
@@ -750,7 +718,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 			fistate=m_pItem->GetStatusUnlocked(nGooddataSize);
 			
 			dbgline;
-			if (fistate>=FIST_ERROR)
+			if (fistate>=FIST_LOCALERROR)
 			{
 				const header &h = m_pItem->GetHeaderUnlocked();
 				g.unLock(); // item lock must be released in order to replace it!
@@ -986,9 +954,7 @@ inline const char * job::BuildAndEnqueHeader(const FiStatus &fistate,
 		}
 		else if (httpstatus >= 300)
 		{
-			bHasSendableData = ( (respHead.h[header::CONTENT_LENGTH]
-			                                 && atoi(respHead.h[header::CONTENT_LENGTH])>0)
-					|| bChunked);
+			bHasSendableData = atoofft(respHead.h[header::CONTENT_LENGTH], 0) || bChunked;
 		}
 	}
 
