@@ -424,4 +424,15 @@ mstring UrlDecode(cmstring &from)
 
 cmstring sEmptyString("");
 
+/*
+int GetSimilarity(cmstring& wanted, cmstring& candidate)
+{
+	const char *w=wanted.c_str(), *c=candidate.c_str();
+	while(w&&c)
+	{
+
+	}
+}
+*/
+
 #endif
