@@ -67,8 +67,10 @@ bool pkgimport::ProcessRegular(const string &sPath, const struct stat &stinfo)
 	
 	if(m_bLookForIFiles)
 	{
+		ProgTell();
+
 		if(0==sPath.compare(CACHE_BASE_LEN, 1, "_"))
-			return true; // not for us, also includes _import
+			return true; // not for us, also excludes _import
 
 		AddIFileCandidate(sPath.substr(CACHE_BASE_LEN));
 	}
@@ -125,7 +127,7 @@ bool pkgimport::ProcessRegular(const string &sPath, const struct stat &stinfo)
 	return true;
 }
 
-bool pkgimport::LinkOrCopy(const MYSTD::string &from, const MYSTD::string &to)
+bool LinkOrCopy(const MYSTD::string &from, const MYSTD::string &to)
 {
 	mkbasedir(to);
 	
@@ -141,6 +143,8 @@ bool pkgimport::LinkOrCopy(const MYSTD::string &from, const MYSTD::string &to)
 		
 		if (!realpath(from.c_str(), buf))
 			return false;
+
+		::unlink(to.c_str());
 		return (0==symlink(buf, to.c_str()));		
 	}
 	else
@@ -161,7 +165,6 @@ bool pkgimport::LinkOrCopy(const MYSTD::string &from, const MYSTD::string &to)
 		if (0==link(from.c_str(), to.c_str()))
 			return true;
 
-		SendChunk("(couldn't link, copying contents)<br>\n");
 		if(!FileCopy(from,to))
 			return false;
 		
@@ -181,7 +184,7 @@ void pkgimport::Action(const string &cmd)
 // TODO: import path from cmd?
 	m_sSrcPath=CACHE_BASE+"_import";
 	
-	SendChunk(string("Importing from ")+m_sSrcPath+" directory, scanning...<br>\n");
+	SendFmt() << "Importing from " << m_sSrcPath << " directory.<br>Scanning local files...<br>";
 	
 	SetCommonUserFlags(cmd);
 	m_bErrAbort=false; // does not f...ing matter, do what we can
@@ -259,6 +262,8 @@ void pkgimport::Action(const string &cmd)
 	else
 		fList << FMTSIG"\n";
 	
+	off_t remaining=0;
+
 	for(tImportMap::const_iterator it=m_importMap.begin();
 	it!=m_importMap.end();it++)
 	{
@@ -275,6 +280,8 @@ void pkgimport::Action(const string &cmd)
 				<< it->first.GetCsAsString() << ENDL
 				<< it->second.sPath.substr(m_sSrcPath.size()+1) <<ENDL
 				<< it->second.mtime	<<ENDL;
+
+		remaining+=it->first.size;
 	}
 
 	// deal with the rest, never double-examine that stuff
@@ -287,9 +294,17 @@ void pkgimport::Action(const string &cmd)
 				<< it->first.GetCsAsString() << ENDL
 				<< it->second.sPath.substr(m_sSrcPath.size()+1) <<ENDL
 				<< it->second.mtime	<<ENDL;
+		remaining+=it->first.size;
 	}
 	fList.flush();
 	fList.close();
+
+	PrintStats("Imported data, per target");
+
+	int n=(m_importMap.size()+m_importRest.size());
+	if(n)
+		SendFmt() << "\n<br>" << n << (n==1 ? " file (" : " files (")
+		<< offttosH(remaining) << ") left behind";
 }
 
 void pkgimport::HandlePkgEntry(const tRemoteFileInfo &entry, bool)
@@ -324,7 +339,9 @@ void pkgimport::HandlePkgEntry(const tRemoteFileInfo &entry, bool)
 	}
 
 	unlink(sDestAbs.c_str());
-	
+
+	// XXX: maybe use inject instead
+
 	if (!LinkOrCopy(sFromAbs, sDestAbs))
 	{
 		SendChunk("<font color=red>ERROR: couldn't link or copy file.</font><br>\n");
@@ -355,6 +372,8 @@ void pkgimport::HandlePkgEntry(const tRemoteFileInfo &entry, bool)
 		return; // junk may remain but that's a job for cleanup later
 	}
 	hit->second.bFileUsed=true;
+
+	SetFlags(m_processedIfile).space+=entry.fpr.size;
 }
 
 
