@@ -91,11 +91,12 @@ void tDeleter::ModContents(mstring & contents, cmstring &cmd)
 	tStrVec files;
 	bool bConfirmMode(stmiss==cmd.find("doDeleteYes"));
 	tSS sHidParms;
+	mstring params(cmd, qpos+1);
 
-	for(tSplitWalk split(cmd.substr(qpos+1), "&"); split.Next();)
+	for(tSplitWalk split(&params, "&"); split.Next();)
 	{
 		char *sep(0);
-		string tok(split);
+		mstring tok(split);
 		if(startsWithSz(tok, "kf")
 				&& strtoul(tok.c_str()+2, &sep, 10)>0
 				&& sep && '=' == *sep)
